#!/usr/bin/env python3
import numpy as np
import os
from isomode.gen_all import run_all

if __name__=="__main__":
    run_all(
        fname='./run.abo_PHBST.nc',  # phonon band netcdf file
        qdict={
            'Gamma': [0.0, 0.0, 0.0],
            'Xy': [0, 0.5, 0],
            'Xx': [0.5, 0.0, 0],
            'M': [0.5, 0.5, 0],
            'Rx': [0.5, 0.0, 0.5],
            'Ry': [0.0, 0.5, 0.5],
            'A': [0.5, 0.5, 0.5],
            'Z': [0, 0, 0.5]
        },  # qpoints in netcdf file
        path='tmp',  # temporary directory, but perhaps you may find things useful in it?
        supercell_matrix=np.eye(3) * 2,  # supercell matrix
        max_freq=0.0,  # maximum frequency. use 0.0 if only unstable mode is required
        amp=0.03,  # amplitude of each mode
        pickle_fname='all_modes.pickle',  # output to this pickle file
        cif_dir='all_modes',  # output cif to this directory
        primitive=True  # whether to make it primitve
)



"""
This file is copied and modified from phonopy!
All the credits go to the original authors of phonopy.
Below is the license of phonopy.

 Copyright (C) 2011 Atsushi Togo
 All rights reserved.

 This file is part of phonopy.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in
   the documentation and/or other materials provided with the
   distribution.

 * Neither the name of the phonopy project nor the names of its
   contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

"""

import numpy as np
import sys
from ase import Atoms
from ase.io import write
import numpy as np


class Supercell(Atoms):
    """Build supercell from supercell matrix
    In this function, unit cell is considered
    [1,0,0]
    [0,1,0]
    [0,0,1].
    Supercell matrix is given by relative ratio, e.g,
    [-1, 1, 1]
    [ 1,-1, 1]  is for FCC from simple cubic.
    [ 1, 1,-1]
    In this case multiplicities of surrounding simple lattice are [2,2,2].

    First, create supercell with surrounding simple lattice.
    Second, trim the surrounding supercell with the target lattice.
    """

    def __init__(self, unitcell, supercell_matrix, symprec=1e-5):
        self._s2u_map = None
        self._u2s_map = None
        self._u2u_map = None
        self._supercell_matrix = np.array(supercell_matrix, dtype='intc')
        self._create_supercell(unitcell, symprec)

    def get_supercell_matrix(self):
        return self._supercell_matrix

    def get_supercell_to_unitcell_map(self):
        return self._s2u_map

    def get_unitcell_to_supercell_map(self):
        return self._u2s_map

    def get_unitcell_to_unitcell_map(self):
        return self._u2u_map

    def _create_supercell(self, unitcell, symprec):
        mat = self._supercell_matrix
        frame = self._get_surrounding_frame(mat)
        sur_cell, u2sur_map = self._get_simple_supercell(frame, unitcell)

        # Trim the simple supercell by the supercell matrix
        trim_frame = np.array([
            mat[0] / float(frame[0]), mat[1] / float(frame[1]),
            mat[2] / float(frame[2])
        ])
        supercell, sur2s_map, mapping_table = trim_cell(trim_frame, sur_cell,
                                                        symprec)

        multi = len(supercell) // len(unitcell)

        if multi != determinant(self._supercell_matrix):
            print("Supercell creation failed.")
            print("Probably some atoms are overwrapped. "
                  "The mapping table is give below.")
            print(mapping_table)
            Atoms.__init__(self)
        else:
            Atoms.__init__(
                self,
                numbers=supercell.get_atomic_numbers(),
                masses=supercell.get_masses(),
                #magmoms=supercell.get_magnetic_moments(),
                scaled_positions=supercell.get_scaled_positions(),
                cell=supercell.get_cell(),
                #pbc=True)
            )
            self._u2s_map = np.arange(len(unitcell)) * multi
            self._u2u_map = dict([(j, i) for i, j in enumerate(self._u2s_map)])
            self._s2u_map = np.array(u2sur_map)[sur2s_map] * multi

    def _get_surrounding_frame(self, supercell_matrix):
        # Build a frame surrounding supercell lattice
        # For example,
        #  [2,0,0]
        #  [0,2,0] is the frame for FCC from simple cubic.
        #  [0,0,2]
        m = np.array(supercell_matrix)
        axes = np.array([[0, 0, 0], m[:, 0], m[:, 1], m[:, 2],
                         m[:, 1] + m[:, 2], m[:, 2] + m[:, 0],
                         m[:, 0] + m[:, 1], m[:, 0] + m[:, 1] + m[:, 2]])
        frame = [max(axes[:, i]) - min(axes[:, i]) for i in (0, 1, 2)]
        return frame

    def _get_simple_supercell(self, multi, unitcell):
        # Scaled positions within the frame, i.e., create a supercell that
        # is made simply to multiply the input cell.
        positions = unitcell.get_scaled_positions()
        numbers = unitcell.get_atomic_numbers()
        masses = unitcell.get_masses()
        #magmoms = unitcell.get_magnetic_moments()
        lattice = unitcell.get_cell()

        atom_map = []
        positions_multi = []
        numbers_multi = []
        if masses is None:
            masses_multi = None
        else:
            masses_multi = []
        #if magmoms is None:
        #    magmoms_multi = None
        #else:
        #    magmoms_multi = []
        for l, pos in enumerate(positions):
            for i in range(multi[2]):
                for j in range(multi[1]):
                    for k in range(multi[0]):
                        positions_multi.append([(pos[0] + k) / multi[0],
                                                (pos[1] + j) / multi[1],
                                                (pos[2] + i) / multi[2]])
                        numbers_multi.append(numbers[l])
                        if masses is not None:
                            masses_multi.append(masses[l])
                        atom_map.append(l)
                        #if magmoms is not None:
                        #    magmoms_multi.append(magmoms[l])

        simple_supercell = Atoms(
            numbers=numbers_multi,
            masses=masses_multi,
            #magmoms=magmoms_multi,
            scaled_positions=positions_multi,
            cell=np.dot(np.diag(multi), lattice),
            #pbc=True
        )

        return simple_supercell, atom_map


class Primitive(Atoms):
    def __init__(self, supercell, primitive_matrix, symprec=1e-5):
        """
        primitive_matrix (3x3 matrix):
        Primitive lattice is given with respect to supercell by
           np.dot(primitive_matrix.T, supercell.get_cell())
        """
        self._primitive_matrix = np.array(primitive_matrix)
        self._symprec = symprec
        self._p2s_map = None
        self._s2p_map = None
        self._p2p_map = None
        self._primitive_cell(supercell)
        self._supercell_to_primitive_map(supercell.get_scaled_positions())
        self._primitive_to_primitive_map()

    def get_primitive_matrix(self):
        return self._primitive_matrix

    def get_primitive_to_supercell_map(self):
        return self._p2s_map

    def get_supercell_to_primitive_map(self):
        return self._s2p_map

    def get_primitive_to_primitive_map(self):
        return self._p2p_map

    def _primitive_cell(self, supercell):
        trimed_cell, p2s_map, mapping_table = trim_cell(
            self._primitive_matrix, supercell, self._symprec)
        Atoms.__init__(
            self,
            numbers=trimed_cell.get_atomic_numbers(),
            masses=trimed_cell.get_masses(),
            #magmoms=trimed_cell.get_magnetic_moments(),
            scaled_positions=trimed_cell.get_scaled_positions(),
            cell=trimed_cell.get_cell(),
            pbc=True)

        self._p2s_map = np.array(p2s_map, dtype='intc')

    def _supercell_to_primitive_map(self, pos):
        inv_F = np.linalg.inv(self._primitive_matrix)
        s2p_map = []
        for i in range(pos.shape[0]):
            s_pos = np.dot(pos[i], inv_F.T)
            for j in self._p2s_map:
                p_pos = np.dot(pos[j], inv_F.T)
                diff = p_pos - s_pos
                diff -= np.rint(diff)
                if (abs(diff) < self._symprec).all():
                    s2p_map.append(j)
                    break
        self._s2p_map = np.array(s2p_map, dtype='intc')

    def _primitive_to_primitive_map(self):
        """
        Mapping table from supercell index to primitive index
        in primitive cell
        """
        self._p2p_map = dict([(j, i) for i, j in enumerate(self._p2s_map)])


def determinant(m):
    return (m[0][0] * m[1][1] * m[2][2] - m[0][0] * m[1][2] * m[2][1] + m[0][1]
            * m[1][2] * m[2][0] - m[0][1] * m[1][0] * m[2][2] + m[0][2] *
            m[1][0] * m[2][1] - m[0][2] * m[1][1] * m[2][0])

def trim_cell(relative_axes, cell, symprec):
    """
    relative_axes: relative axes to supercell axes
    Trim positions outside relative axes

    """
    positions = cell.get_scaled_positions()
    numbers = cell.get_atomic_numbers()
    masses = cell.get_masses()
    #magmoms = cell.get_magnetic_moments()
    lattice = cell.get_cell()
    trimed_lattice = np.dot(relative_axes.T, lattice)

    trimed_positions = []
    trimed_numbers = []
    if masses is None:
        trimed_masses = None
    else:
        trimed_masses = []
    #if magmoms is None:
    #    trimed_magmoms = None
    #else:
    #    trimed_magmoms = []
    extracted_atoms = []

    positions_in_new_lattice = np.dot(positions,
                                      np.linalg.inv(relative_axes).T)
    positions_in_new_lattice -= np.floor(positions_in_new_lattice)
    trimed_positions = np.zeros_like(positions_in_new_lattice)
    num_atom = 0

    mapping_table = np.arange(len(positions), dtype='intc')
    symprec2 = symprec**2
    for i, pos in enumerate(positions_in_new_lattice):
        is_overlap = False
        if num_atom > 0:
            diff = trimed_positions[:num_atom] - pos
            diff -= np.rint(diff)
            # Older numpy doesn't support axis argument.
            # distances = np.linalg.norm(np.dot(diff, trimed_lattice), axis=1)
            # overlap_indices = np.where(distances < symprec)[0]
            distances2 = np.sum(np.dot(diff, trimed_lattice)**2, axis=1)
            overlap_indices = np.where(distances2 < symprec2)[0]
            if len(overlap_indices) > 0:
                is_overlap = True
                mapping_table[i] = extracted_atoms[overlap_indices[0]]

        if not is_overlap:
            trimed_positions[num_atom] = pos
            num_atom += 1
            trimed_numbers.append(numbers[i])
            if masses is not None:
                trimed_masses.append(masses[i])
            #if magmoms is not None:
            #    trimed_magmoms.append(magmoms[i])
            extracted_atoms.append(i)

    trimed_cell = Atoms(
        numbers=trimed_numbers,
        masses=trimed_masses,
        #magmoms=trimed_magmoms,
        scaled_positions=trimed_positions[:num_atom],
        cell=trimed_lattice, )

    return trimed_cell, extracted_atoms, mapping_table




if __name__ == '__main__':
    test()




import numpy as np
import sys
# from phonopy.structure.cells import get_supercell
from ase import Atoms
from ase.io import write

import numpy as np
import copy
import pyDFTutils.perovskite.perovskite_mode as perovskite_mode
from pyDFTutils.perovskite.perovskite_mode import Gamma_modes

import spglib.spglib
#from phonopy.structure.atoms import PhonopyAtoms as Atoms
from pyDFTutils.perovskite.cubic_perovskite import gen_primitive
from pyDFTutils.ase_utils import vesta_view
from isomode.frozen_mode import distorted_cell, default_mod_func
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
from functools import partial


def gen_domain(
        name,
        cell=[3.9, 3.9, 3.9],
        supercell_matrix=[[1, -1, 0], [1, 1, 0], [0, 0, 2]],
        modes=dict(
            R2_m_O1=(0.0, default_mod_func,
                     0.0),  # R2-[O1:c:dsp]A2u(a), O, breathing
            R3_m_O1=(
                0.0, default_mod_func, 0.0
            ),  # R3-[O1:c:dsp]A2u(a), O JT inplane-stagger, out-of-plane antiphase
            R3_m_O2=(
                0.0, default_mod_func, 0.0
            ),  # R3-[O1:c:dsp]A2u(b), O, out-of-plane-stagger, inplane antiphase, Unusual.
            R4_m_A1=(0.0, default_mod_func,
                     0.0),  # R4-[Nd1:a:dsp]T1u(a), A , Unusual
            R4_m_A2=(0.0, default_mod_func,
                     0.0),  # R4-[Nd1:a:dsp]T1u(b), A, Unusual
            R4_m_A3=(0.0, default_mod_func,
                     0.0),  # R4-[Nd1:a:dsp]T1u(c), A, Unusual
            R4_m_O1=(0.0, default_mod_func,
                     0.0),  # R4-[O1:c:dsp]Eu(a), O, Unusual
            R4_m_O2=(0.0, default_mod_func,
                     0.0),  # R4-[O1:c:dsp]Eu(b), O, Unusual
            R4_m_O3=(0.0, default_mod_func,
                     0.0),  # R4-[O1:c:dsp]Eu(c), O, Unusual
            R5_m_O1=(0.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(a), O  a-
            R5_m_O2=(0.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(b), O  b-
            R5_m_O3=(0.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(c), O  c-
            #X3_m_A1=(0.0, default_mod_func,
            #         0.0),  # X3-[Nd1:a:dsp]T1u(a), What's this..
            #X3_m_O1=(0.0, default_mod_func, 0.0),  # X3-[O1:c:dsp]A2u(a)

            #X5_m_A1=(0.0, default_mod_func, 0.0),  # [Nd1:a:dsp]T1u(a), A , Antiferro mode
            #X5_m_A2=(0.0, default_mod_func, 0.0),  # [Nd1:a:dsp]T1u(b), A , save as above
            #X5_m_O1=(0.0, default_mod_func, 0.0),  # [Nd1:a:dsp]T1u(a), O , Antiferro mode
            #X5_m_O2=(0.0, default_mod_func, 0.0),  # [Nd1:a:dsp]T1u(b), O , same as above
            #M2_p_O1=(0.0, default_mod_func, 0.0),  # M2+[O1:c:dsp]Eu(a), O, In phase rotation c+
            Z5_m_A1=(0.0, default_mod_func,
                     0.0),  # [Nd1:a:dsp]T1u(a), A , Antiferro mode
            Z5_m_A2=(0.0, default_mod_func,
                     0.0),  # [Nd1:a:dsp]T1u(b), A , save as above
            Z5_m_O1=(0.0, default_mod_func,
                     0.0),  # [Nd1:a:dsp]T1u(a), O , Antiferro mode
            Z5_m_O2=(0.0, default_mod_func,
                     0.0),  # [Nd1:a:dsp]T1u(b), O , same as above
            M2_p_O1=(0.0, default_mod_func,
                     0.0),  # M2+[O1:c:dsp]Eu(a), O, In phase rotation
            M3_p_O1=(0.0, default_mod_func,
                     0.0),  # M3+[O1:c:dsp]A2u(a), O, D-type JT inplane stagger
            M5_p_O1=(0.0, default_mod_func,
                     0.0),  # M5+[O1:c:dsp]Eu(a), O, Out of phase tilting
            M5_p_O2=(0.0, default_mod_func,
                     0.0),  # M5+[O1:c:dsp]Eu(a), O, Out of phase tilting
            M4_p_O1=(
                0.0,
                default_mod_func, 0.0
            ),  # M4+[O1:c:dsp]A2u(a), O, in-plane-breathing (not in P21/c)
            G_Ax=(0.0, default_mod_func, 0.0),
            G_Ay=(0.0, default_mod_func, 0.0),
            G_Az=(0.0, default_mod_func, 0.0),
            G_Sx=(0.0, default_mod_func, 0.0),
            G_Sy=(0.0, default_mod_func, 0.0),
            G_Sz=(0.0, default_mod_func, 0.0),
            G_Axex=(0.0, default_mod_func, 0.0),
            G_Axey=(0.0, default_mod_func, 0.0),
            G_Axez=(0.0, default_mod_func, 0.0),
            G_Lx=(0.0, default_mod_func, 0.0),
            G_Ly=(0.0, default_mod_func, 0.0),
            G_Lz=(0.0, default_mod_func, 0.0),
            G_G4x=(0.0, default_mod_func, 0.0),
            G_G4y=(0.0, default_mod_func, 0.0),
            G_G4z=(0.0, default_mod_func, 0.0),
        ), return_nondistorted=True):
    atoms = gen_primitive(name=name, mag_order='PM', latticeconstant=cell[0])
    spos = atoms.get_scaled_positions()
    atoms.set_cell(cell)
    atoms.set_scaled_positions(spos)
    dcell = distorted_cell(atoms, supercell_matrix=supercell_matrix)
    eigvec = np.zeros(15)

    mode_dict = {
        'R2_m_O1': perovskite_mode.R2p,
        'R3_m_O1': perovskite_mode.
        R12p_1,  # R3-[O1:c:dsp]A2u(a), O JT inplane-stagger, out-of-plane antiphase
        'R3_m_O2': perovskite_mode.
        R12p_2,  # R3-[O1:c:dsp]A2u(b), O, out-of-plane-stagger, inplane antiphase
        'R4_m_A1': perovskite_mode.R15_1,  # R4-[Nd1:a:dsp]T1u(a), A
        'R4_m_A2': perovskite_mode.R15_2,  # R4-[Nd1:a:dsp]T1u(b), A
        'R4_m_A3': perovskite_mode.R15_3,  # R4-[Nd1:a:dsp]T1u(c), A
        'R4_m_O1': perovskite_mode.R15_4,  # R4-[O1:c:dsp]Eu(a), O
        'R4_m_O2': perovskite_mode.R15_5,  # R4-[O1:c:dsp]Eu(b), O
        'R4_m_O3': perovskite_mode.R15_6,  # R4-[O1:c:dsp]Eu(c), O
        'R5_m_O1': perovskite_mode.
        R25_1,  # R5-[O1:c:dsp]Eu(a), O, out-of-phase rotation a-
        'R5_m_O2': perovskite_mode.R25_2,  # R5-[O1:c:dsp]Eu(b), O, b-
        'R5_m_O3': perovskite_mode.
        R25_3,  # R5-[O1:c:dsp]Eu(c), O, c-. For Pnma. Do not use.

        #'X3_m_A1':perovskite_mode., # X3-[Nd1:a:dsp]T1u(a), What's this..
        #'X3_m_O1':perovskite_mode., # X3-[O1:c:dsp]A2u(a)
        'Z5_m_A1':
        perovskite_mode.Z5p_1,  # [Nd1:a:dsp]T1u(a), A , Antiferro mode
        'Z5_m_A2':
        perovskite_mode.Z5p_2,  # [Nd1:a:dsp]T1u(b), A , save as above
        'Z5_m_O1':
        perovskite_mode.Z5p_3,  # [Nd1:a:dsp]T1u(b), O , same as above
        'Z5_m_O2':
        perovskite_mode.Z5p_4,  # [Nd1:a:dsp]T1u(b), O , same as above
        'M2_p_O1':
        perovskite_mode.M3,  # M2+[O1:c:dsp]Eu(a), O, In phase rotation
        'M3_p_O1': perovskite_mode.
        M2,  # M3+[O1:c:dsp]A2u(a), O, D-type JT inplane stagger
        'M5_p_O1':
        perovskite_mode.M5_1,  # M5+[O1:c:dsp]Eu(a), O, Out of phase tilting
        'M5_p_O2': perovskite_mode.
        M5_2,  # M5+[O1:c:dsp]Eu(b), O, Out of phase tilting, -above
        'M4_p_O1': perovskite_mode.
        M4,  # M4+[O1:c:dsp]A2u(a), O, in-plane-breathing (not in P21/c)
    }

    # add Gamma modes to mode_dict
    Gamma_mode_dict = Gamma_modes(atoms.get_chemical_symbols())
    mode_dict.update(Gamma_mode_dict)

    mode_disps = {}
    qdict = {
        'G': [0, 0, 0],
        #'X':[0,0.0,0.5],
        'M': [0.5, 0.5, 0],
        'R': [0.5, 0.5, 0.5],
        'Z': [0.0, 0.0, 0.5]
    }
    disps = 0.0  #np.zeros(3,dtype='complex128')
    for key, val in modes.items():
        name = key
        amp, mod_func, argument = val
        eigvec = np.array(mode_dict[name])
        disp = dcell._get_displacements(
            eigvec=eigvec,
            q=qdict[name[0]],
            amplitude=amp,
            argument=0,
            mod_func=mod_func)
        disps += disp

    nondistorted = dcell._supercell
    nondistorted.set_pbc(True)
    newcell = dcell._get_cell_with_modulation(disps)
    newcell = Atoms(newcell)
    print(spglib.get_spacegroup(newcell))
    if return_nondistorted:
        return newcell, nondistorted
    else:
        return newcell


def isotropy_normfactor(scell, sc_mat, disps):
    """
    pcell: primitive cell parameter. 3*3 matrix
    sc_mat: primitive->supercell transform matrix. 3*3 matrix
    disps: list of vectors defining displacements.
    """
    sum = 0.0
    for disp in disps:
        sum += (np.linalg.norm(np.dot(scell, disp)))**2
        norm_factor = 1.0 / np.sqrt(sum)
    return norm_factor


def conv_circ(signal, ker):
    '''
        signal: real 1D array
        ker: real 1D array
        signal and ker must have same shape
    '''
    return np.real(np.fft.ifft(np.fft.fft(signal) * np.fft.fft(ker)))


from scipy.ndimage import convolve


def smooth_domain_func(R, width, ncell, pos, dim, d0_value=1.0, d1_value=-1.0):
    """
    This funcunction create a smoothed step function in x, y, or z direction. The size is ncell. The value is 1 in a range (0~pos*ncell), and low_value in other part.
    ncell: number of cells along one direction
    width: domain wall width in the unit of cell.
    pos: position of the domain wall . Note that there are two domains, so there is alwasys another
         domain wall at 0. 0<pos<1.
    dim: 0,1,2-> x, y, or z direction.
    d0_value: amplitude coefficient in first domain
    d1_value: amplitude coefficient in second domain
    """
    r = 100
    x0 = np.linspace(0, ncell, ncell * r, endpoint=True)
    pos = int(np.floor(ncell * r * pos))
    y0 = np.zeros_like(x0)
    y0[:pos] = d0_value
    y0[pos:] = d1_value
    y0 = convolve(y0, [1.0 / r / width] * r * width, mode='wrap')
    return interp1d(x0, y0)(R[:, dim])


def mod_func_1d(R, width, ncell, pos, dim):
    return smooth_domain_func(width, ncell)(R[:, dim])


mod_func_x = partial(smooth_domain_func, width=4, ncell=16, pos=0.5, dim=0)
mod_func_y = partial(smooth_domain_func, width=4, ncell=16, pos=0.5, dim=1)
mod_func_z = partial(smooth_domain_func, width=4, ncell=16, pos=0.5, dim=2)

mod_func_x10 = partial(
    smooth_domain_func, width=4, ncell=16, pos=0.5, dim=0, d1_value=0.0)
mod_func_y10 = partial(
    smooth_domain_func, width=4, ncell=16, pos=0.5, dim=1, d1_value=0.0)
mod_func_z10 = partial(
    smooth_domain_func, width=4, ncell=16, pos=0.5, dim=2, d1_value=0.0)

mod_func_x01 = partial(
    smooth_domain_func,
    width=4,
    ncell=16,
    pos=0.5,
    dim=0,
    d0_value=0.0,
    d1_value=1.0)
mod_func_y01 = partial(
    smooth_domain_func,
    width=4,
    ncell=16,
    pos=0.5,
    dim=1,
    d0_value=0.0,
    d1_value=1.0)
mod_func_z01 = partial(
    smooth_domain_func,
    width=4,
    ncell=16,
    pos=0.5,
    dim=2,
    d0_value=0.0,
    d1_value=1.0)


def antipolar_180_x():
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.7, 3.7, 3.7],
        supercell_matrix=[[16, 0, 0], [0, 2, 0], [0, 0, 2]],
        modes=dict(
            Z5_m_A1=(1.5, mod_func_x,
                     0.0),  # [Nd1:a:dsp]T1u(a), A , Antiferro mode
            Z5_m_A2=(1.5, mod_func_x,
                     0.0),  # [Nd1:a:dsp]T1u(b), A , save as above
            Z5_m_O1=(2.0, mod_func_x,
                     0.0),  # [Nd1:a:dsp]T1u(a), O , Antiferro mode
            Z5_m_O2=(2.0, mod_func_x,
                     0.0))  # [Nd1:a:dsp]T1u(b), O , same as above

    )
    atoms.set_pbc([True, True, True])
    write('antipolar_180domain_x_nondistorted.cif', orig)
    write('antipolar_180domain_x.cif', atoms)
    vesta_view(atoms)


def antipolar_180_z():
    # amplitude, mod_func
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.7, 3.7, 3.7],
        supercell_matrix=[[2, 0, 0], [0, 2, 0], [0, 0, 16]],
        modes=dict(
            R5_m_O1=(2.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(a), O  a-
            R5_m_O2=(2.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(b), O  b-
            R5_m_O3=(2.0, default_mod_func, 0.0),  # R5-[O1:c:dsp]Eu(c), O  c-
 
            Z5_m_A1=(1.5, mod_func_z,
                     0.0),  # [Nd1:a:dsp]T1u(a), A , Antiferro mode
            Z5_m_A2=(1.5, mod_func_z,
                     0.0),  # [Nd1:a:dsp]T1u(b), A , save as above
            Z5_m_O1=(2.0, mod_func_z,
                     0.0),  # [Nd1:a:dsp]T1u(a), O , Antiferro mode
            Z5_m_O2=(2.0, mod_func_z, 0.0),

        ))  
    atoms.set_pbc([True, True, True])

    write('antipolar_180domain_z_nondistorted.cif', orig)
    write('antipolar_180domain_z.cif', atoms)
    vesta_view(atoms)


def polar_180_x_longi():
    # amplitude, mod_func
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.901, 4.128, 3.901],
        supercell_matrix=[[16, 0, 0], [0, 1, 0], [0, 0, 1]],
        modes=dict(
            G_Ax=(0.0, default_mod_func, 0.0),
            G_Ay=(0.0, default_mod_func, 0.0),
            G_Az=(0.0, default_mod_func, 0.0),
            G_Sx=(0.0, default_mod_func, 0.0),
            G_Sy=(1.0, mod_func_x, 0.0),
            G_Sz=(0.0, mod_func_x, 0.0),
            G_Axex=(0.0, default_mod_func, 0.0),
            G_Axey=(0.0, default_mod_func, 0.0),
            G_Axez=(0.0, default_mod_func, 0.0),
            G_Lx=(0.0, default_mod_func, 0.0),
            G_Ly=(2.0, mod_func_x, 0.0),
            G_Lz=(0.0, mod_func_x, 0.0),
            G_G4x=(0.0, default_mod_func, 0.0),
            G_G4y=(0.0, default_mod_func, 0.0),
            G_G4z=(0.0, default_mod_func, 0.0),
        ))
    atoms.set_pbc([True, True, True])
    write('polar_180domain_nondistored.cif', orig)
    write('polar_180domain.cif', atoms)
    vesta_view(atoms)


def polar_180_x_trans():
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.901, 3.7, 3.7],
        supercell_matrix=[[16, 0, 0], [0, 2, 0], [0, 0, 2]],
        modes=dict(
            G_Sx=(3.0, mod_func_x, 0.0),
        ))
    atoms.set_pbc(True)
    write("polar_180_x_nondistorted.cif",atoms)
    write("polar_180_x.cif",atoms)
    vesta_view(atoms)

def polar_180_x_trans_phonon():
    phon = PhonMode.loadDDB('PbTiO3.ddb')
    atoms= phon.gen_domain(
        supercell_matrix=[[16, 0, 0], [0, 2, 0], [0, 0, 2]],
        modes={((0.0,0.0,0.0), 1): (3.0, mod_func_x, 0.0)}
        )
    atoms.set_pbc(True)
    write("polar_180_x_phonon.cif",atoms)
    vesta_view(atoms)


def polar_90_xy_trans():
    # amplitude, mod_func
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.7, 3.7, 3.7],
        supercell_matrix=[[16, 0, 0], [0, 2, 0], [0, 0, 2]],
        modes=dict(
            G_Ax=(0.0, default_mod_func, 0.0),
            G_Ay=(0.0, default_mod_func, 0.0),
            G_Az=(0.0, default_mod_func, 0.0),
            G_Sx=(3.0, mod_func_x10, 0.0),
            G_Sy=(3.0, mod_func_x01, 0.0),
            G_Sz=(0.0, default_mod_func, 0.0),
            G_Axex=(0.0, default_mod_func, 0.0),
            G_Axey=(0.0, default_mod_func, 0.0),
            G_Axez=(0.0, default_mod_func, 0.0),
            G_Lx=(0.0, default_mod_func, 0.0),
            G_Ly=(0.0, default_mod_func, 0.0),
            G_Lz=(0.0, default_mod_func, 0.0),
            G_G4x=(0.0, default_mod_func, 0.0),
            G_G4y=(0.0, default_mod_func, 0.0),
            G_G4z=(0.0, default_mod_func, 0.0),
        ))
    atoms.set_pbc(True)
    write("polar_90_nondistorted.cif", orig)
    write("polar_90.cif", atoms)
    vesta_view(atoms)

def cyc_func_xy(R):
    return np.sin(R[:, 1]*4*np.pi/16.0) #+ np.cos(R[:, 1]*4*np.pi/16.0)


def cyc_func_yx(R):
    return np.cos(R[:, 0]*4*np.pi/16.0) #+ np.sin(R[:, 1]*4*np.pi/16.0)


def polar_cycle_xy():
    # amplitude, mod_func
    atoms, orig = gen_domain(
        name='PbTiO3',
        cell=[3.7, 3.7, 3.7],
        supercell_matrix=[[16, 0, 0], [0, 16, 0], [0, 0, 2]],
        modes=dict(
            G_Ax=(0.0, default_mod_func, 0.0),
            G_Ay=(0.0, default_mod_func, 0.0),
            G_Az=(0.0, default_mod_func, 0.0),
            G_Sx=(9.0, cyc_func_xy, 0.0),
            G_Sy=(9.0, cyc_func_yx, 0.0),
            G_Sz=(0.0, default_mod_func, 0.0),
            G_Axex=(0.0, default_mod_func, 0.0),
            G_Axey=(0.0, default_mod_func, 0.0),
            G_Axez=(0.0, default_mod_func, 0.0),
            G_Lx=(0.0, default_mod_func, 0.0),
            G_Ly=(0.0, default_mod_func, 0.0),
            G_Lz=(0.0, default_mod_func, 0.0),
            G_G4x=(0.0, default_mod_func, 0.0),
            G_G4y=(0.0, default_mod_func, 0.0),
            G_G4z=(0.0, default_mod_func, 0.0),
        ))
    write("polar_cycle_nondistorted.cif", orig)
    write("polar_cycle.cif", atoms)
    vesta_view(atoms)



antipolar_180_x()
antipolar_180_z()
polar_180_x_longi()
polar_90_xy_trans()
polar_cycle_xy()



#!/usr/bin/env python
import numpy as np
from collections import namedtuple, OrderedDict
from ase.data import atomic_numbers, atomic_masses

nmode = namedtuple('nmode', [
    'Ax', 'Ay', 'Az', 'Bx', 'By', 'Bz', 'O1x', 'O1y', 'O1z', 'O2x', 'O2y',
    'O2z', 'O3x', 'O3y', 'O3z'
])  # Note: A, B, Oz, Ox, Oy

IR_dict = OrderedDict()

zvec = nmode._make([0.0] * 15)

# Gamma point
D1_1 = zvec._replace(Ay=1)
D1_2 = zvec._replace(By=1)
D1_3 = zvec._replace(O3y=1)
D1_4 = zvec._replace(O1y=1, O2y=1)

D2 = zvec._replace(O1y=1, O2y=-1)

D5_1 = zvec._replace(Ax=1)
D5_2 = zvec._replace(Bx=1)
D5_3 = zvec._replace(O1x=1)
D5_4 = zvec._replace(O2x=1)
D5_5 = zvec._replace(O3x=1)

D5_6 = zvec._replace(Az=1)
D5_7 = zvec._replace(Bz=1)
D5_8 = zvec._replace(O1z=1)
D5_9 = zvec._replace(O2z=1)
D5_10 = zvec._replace(O3z=1)

IR_dict['Gamma'] = {
    D1_1: '$\Delta_1$',
    D1_2: '$\Delta_1$',
    D1_3: '$\Delta_1$',
    D1_4: '$\Delta_1$',
    D2: '$\Delta_2$',
    D5_1: '$\Delta_5$',
    D5_2: '$\Delta_5$',
    D5_3: '$\Delta_5$',
    D5_4: '$\Delta_5$',
    D5_5: '$\Delta_5$',
    D5_6: '$\Delta_5$',
    D5_7: '$\Delta_5$',
    D5_8: '$\Delta_5$',
    D5_9: '$\Delta_5$',
    D5_10: '$\Delta_5$',
}

# Z (0,0,1/2) for generating distorted structures.
Z2p_1 = zvec._replace(Az=1)
Z2p_2 = zvec._replace(O1z=1)

Z5p_1 = zvec._replace(Ax=1)
Z5p_2 = zvec._replace(Ay=1)
Z5p_3 = zvec._replace(O1x=1)
Z5p_4 = zvec._replace(O1y=1)

# X point
X1_1 = zvec._replace(By=1)
X1_2 = zvec._replace(O1y=1, O2y=1)

X2p_1 = zvec._replace(Ay=1)
X2p_2 = zvec._replace(O3y=1)

X3 = zvec._replace(O1y=1, O2y=-1)

X5_1 = zvec._replace(Bx=1)
X5_2 = zvec._replace(Bz=1)
X5_3 = zvec._replace(O1x=1)
X5_4 = zvec._replace(O1z=1)
X5_5 = zvec._replace(O2x=1)
X5_6 = zvec._replace(O2z=1)

X5p_1 = zvec._replace(Ax=1)
X5p_2 = zvec._replace(Az=1)
X5p_3 = zvec._replace(O3x=1)
X5p_4 = zvec._replace(O3z=1)

IR_dict['X'] = {
    X1_1: '$M_1$',
    X1_2: '$M_1$',
    X2p_1: '$M_2\prime$',
    X2p_2: '$M_2\prime$',
    X3: '$M_3$',
    X5_1: '$M_5$',
    X5_2: '$M_5$',
    X5_3: '$M_5$',
    X5_4: '$M_5$',
    X5_5: '$M_5$',
    X5_6: '$M_5$',
    X5p_1: '$M_5\prime$',
    X5p_2: '$M_5\prime$',
    X5p_3: '$M_5\prime$',
    X5p_4: '$M_5\prime$',
}
# M point

M = nmode._make([0.0] * 15)
M1 = M._replace(O3x=1, O2y=1)

M2 = M._replace(O2x=1, O3y=-1)

M3 = M._replace(O3x=1, O2y=-1)

M4 = M._replace(O2x=1, O3y=1)

M2p = M._replace(Az=1)

M3p_1 = M._replace(Bz=1)

M3p_2 = M._replace(O1z=1)

M5_1 = M._replace(O3z=1)

M5_2 = M._replace(O2z=1)

M5p_1 = M._replace(Bx=1)

M5p_2 = M._replace(By=1)

M5p_3 = M._replace(Ay=1)

M5p_4 = M._replace(Ax=1)

M5p_5 = M._replace(O1x=1)

M5p_6 = M._replace(O1y=1)

IR_dict['M'] = {
    M1: '$M_1$',
    M2: '$M_2$',
    M3: '$M_3$',
    M4: '$M_4$',
    M2p: '$M_2\prime$',
    M3p_1: '$M_3\prime$',
    M3p_2: '$M_3\prime$',
    M5_1: '$M_5$',
    M5_2: '$M_5$',
    M5p_1: '$M_5\prime$',
    M5p_2: '$M_5\prime$',
    M5p_3: '$M_5\prime$',
    M5p_4: '$M_5\prime$',
    M5p_5: '$M_5\prime$',
    M5p_6: '$M_5\prime$',
}
# R point

# Breathing mode
R = nmode._make([0.0] * 15)
R2p = R._replace(O1z=1, O2x=1, O3y=1)

R12p_1 = R._replace(O1z=1, O3y=1, O2x=-2)

R12p_2 = R._replace(O1z=1, O3y=-1)

R25_1 = R._replace(O1y=1, O3z=-1)

R25_2 = R._replace(O1x=1, O2z=-1)

R25_3 = R._replace(O3x=1, O2y=-1)

R25p_1 = R._replace(Bx=1)

R25p_2 = R._replace(By=1)

R25p_3 = R._replace(Bz=1)

R15_1 = R._replace(Ax=1)

R15_2 = R._replace(Ay=1)

R15_3 = R._replace(Az=1)

R15_4 = R._replace(O1y=1, O3z=1)

R15_5 = R._replace(O1x=1, O2z=1)

R15_6 = R._replace(O3x=1, O2y=1)

IR_dict['R'] = {
    R2p: r'$\Gamma_2\prime$',
    R12p_1: r'$\Gamma_{12}\prime$',
    R12p_2: r'$\Gamma_{12}\prime$',
    R25_1: r'$\Gamma_{25}$',
    R25_2: r'$\Gamma_{25}$',
    R25_3: r'$\Gamma_{25}$',
    R25p_1: r'$\Gamma_{25}\prime$',
    R25p_2: r'$\Gamma_{25}\prime$',
    R25p_3: r'$\Gamma_{25}\prime$',
    R15_1: r'$\Gamma_{15}$',
    R15_2: r'$\Gamma_{15}$',
    R15_3: r'$\Gamma_{15}$',
    R15_4: r'$\Gamma_{15}$',
    R15_5: r'$\Gamma_{15}$',
    R15_6: r'$\Gamma_{15}$',
}


def label_zone_boundary(qname,
                        phdisp=None,
                        masses=None,
                        evec=None,
                        notation='IR'):
    IR_translation = {}

    IR_translation['Gamma'] = {
        '$\Delta_1$': r'$\Gamma_4^-$',
        '$\Delta_2$': r'',
        '$\Delta_5$': r'',
    }

    IR_translation['R'] = {
        r'$\Gamma_2\prime$': '$R_2^-$',
        r'$\Gamma_{12}\prime$': '$R_3^-$',
        r'$\Gamma_{25}$': '$R_5^-$',
        r'$\Gamma_{25}\prime$': '$R_5^+$',
        r'$\Gamma_{15}$': '$R_4^-$',
    }

    IR_translation['X'] = {
        '$M_1$': '$X_1^+$',
        '$M_2\prime$': '$X_3^-$',
        '$M_3$': '$X_2^+$',
        '$M_5$': '$X_5^+$',
        '$M_5\prime$': '$X_5^-$',
    }

    IR_translation['M'] = {
        '$M_1$': '$M_1^+$',
        '$M_2$': '$M_3^+$',
        '$M_3$': '$M_2^+$',
        '$M_4$': '$M_4^+$',
        '$M_2\prime$': '$M_3^-$',
        '$M_3\prime$': '$M_2^-$',
        '$M_5$': '$M_5^+$',
        '$M_5\prime$': '$M_5^-$',
    }

    if phdisp is not None or masses is not None:
        evec = np.array(phdisp) * np.sqrt(np.kron(masses, [1, 1, 1]))
        evec = np.real(evec) / np.linalg.norm(evec)

    mode = None
    for m in IR_dict[qname]:
        mvec = np.real(m)
        mvec = mvec / np.linalg.norm(mvec)
        evec = evec / np.linalg.norm(evec)
        p = np.abs(np.dot(np.real(evec), mvec))
        if p > 0.4:  #1.0 / np.sqrt(2):
            print("-------------")
            #print("Found! p= %s" % p)
            #print("eigen vector: ", nmode._make(mvec))
            if notation.lower() == 'cowley':
                mode = IR_dict[qname][m]
            elif notation.upper() == 'IR':
                #print(IR_translation[qname])
                mode = IR_translation[qname][IR_dict[qname][m]]
            elif notation.lower() == 'both':
                mode = (IR_dict[qname][m],
                        IR_translation[qname][IR_dict[qname][m]])
            else:
                raise ValueError('notation should be Cowley|IR|both')
            #print("mode: ", mode, m)
    if mode is None:
        print("==============")
        print("eigen vector: ", nmode._make(evec))
    return mode


def label_Gamma(phdisp=None, masses=None, evec=None):
    if phdisp is not None and masses is not None:
        evec = np.array(phdisp) * np.sqrt(np.kron(masses, [1, 1, 1]))
        evec = np.real(evec) / np.linalg.norm(evec)

    basis_dict = mass_to_Gamma_basis_3d(masses, eigen_type='eigen_vector')
    mode = None

    for m in basis_dict:
        mvec = np.real(m)
        mvec = mvec / np.linalg.norm(mvec)
        evec = evec / np.linalg.norm(evec)
        p = np.abs(np.dot(np.real(evec), mvec))
        if p > 0.5:  #1.0 / np.sqrt(2):
            mode = basis_dict[m]
    if mode is None:
        print("==============")
        print("eigen vector: ", nmode._make(evec))
    return mode


def mass_to_Gamma_basis(masses, eigen_type='eigen_vector'):
    """
    return the slater-last-axe basis.
    type: eigenvector |eigendisplacement
    Notice that the sequence of the atoms are A, B, O_inplane, O_inplane, O_out_of_plane.
    """
    masses = np.sqrt(masses)
    A, B, O = masses[0], masses[1], masses[2]

    # acoustic
    s = np.array([1, 1, 1, 1, 1])
    acoustic = s / np.linalg.norm(s)

    #slater: A0, B+, O//- O|_-
    s = np.array([0, O * 3, -B, -B, -B])
    slater = s / np.linalg.norm(s)

    # axe
    s = np.array([0, 0, 2, -1, -1])
    axe = s / np.linalg.norm(s)

    # last :  A- B- O+
    s = np.array([B + O * 3, -A, -A, -A, -A])
    last = s - np.dot(s, slater) * slater
    last = last / np.linalg.norm(last)

    # Gamma4 :
    s = np.array([0, 0, 0, 1, -1])
    g4 = s / np.linalg.norm(s)

    if eigen_type == 'eigen_vector':
        slater = slater * masses
        slater = slater / np.linalg.norm(slater)
        axe = axe * masses
        axe = axe / np.linalg.norm(axe)
        last = last * masses
        last = last / np.linalg.norm(last)
        acoustic = acoustic * masses
        acoustic = acoustic / np.linalg.norm(acoustic)
        g4 = g4 * masses
        g4 = g4 / np.linalg.norm(g4)

    #return acoustic, slater, axe, last, g4
    return {
        'acoustic': acoustic,
        'slater': slater,
        'axe': axe,
        'last': last,
        'g4': g4
    }


def mass_to_Gamma_basis_3d(masses, eigen_type='eigen_vector'):
    evecs = mass_to_Gamma_basis(masses, eigen_type=eigen_type)
    M = nmode._make([0.0] * 15)

    a = evecs['acoustic']
    s = evecs['slater']
    ax = evecs['axe']
    l = evecs['last']
    g = evecs['g4']

    # acoustic mode.
    v = np.zeros(15, dtype=float)
    v[0::3] = a
    Ax = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[1::3] = a
    Ay = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[2::3] = a
    Az = nmode._make(v)

    # Slater mode
    v = np.zeros(15, dtype=float)
    v[0::3] = s
    Sx = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[1::3] = s
    Sy = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[2::3] = s
    Sz = nmode._make(v)

    # Axe mode
    r1, r2 = ax[2], ax[3]  # O1, O2 -- Oz Ox
    Axex = M._replace(O1x=r2, O2x=r1, O3x=r2)
    Axey = M._replace(O1y=r2, O2y=r2, O3y=r1)
    Axez = M._replace(O1z=r1, O2z=r2, O3z=r2)

    # Last mode
    v = np.zeros(15, dtype=float)
    v[0::3] = l
    Lx = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[1::3] = l
    Ly = nmode._make(v)

    v = np.zeros(15, dtype=float)
    v[2::3] = l
    Lz = nmode._make(v)

    # Gamma4 silent mode
    r1, r2 = g[3], g[4]
    G4x = M._replace(O1x=r1, O3x=r2)  # Ox, Oy in z direction
    G4y = M._replace(O1y=r1, O2y=r2)  # Ox, Oz in y direction
    G4z = M._replace(O2z=r1, O3z=r2)  # Ox, Oy in z direction

    mode_dict = {
        Ax: 'Ax',
        Ay: 'Ay',
        Az: 'Az',
        Sx: 'Sx',
        Sy: 'Sy',
        Sz: 'Sz',
        Axex: 'Axex',
        Axey: 'Axey',
        Axez: 'Axez',
        Lx: 'Lx',
        Ly: 'Ly',
        Lz: 'Lz',
        G4x: 'G4x',
        G4y: 'G4y',
        G4z: 'G4z'
    }
    return mode_dict


def elem_to_Gamma_basis(elems, eigen_type='eigen_displacement'):
    """
    Input A, B, X and get Slater, axe, Last mode in perovskite structure.
    eigen_type: eigen_displacement or eigen_vector. Default is eigen displacement.
    """
    masses = [atomic_masses[atomic_numbers[elem]] for elem in elems]
    return mass_to_Gamma_basis(masses, eigen_type=eigen_type)

def test():
    from ase import Atoms
    atoms=Atoms('BaTiO3')
    masses=atoms.get_masses()
    m=mass_to_Gamma_basis_3d(masses, eigen_type='eigen_vector')
    for k in m:
        print(m[k],k)
#test()



import numpy as np
import sys
from ase import Atoms
from ase.io import write
import numpy as np
import copy
import isomode.perovskite_mode as perovskite_mode
import spglib.spglib
from isomode.extern.cells import Supercell, Primitive

def default_mod_func(R):
    return 1.0

class distorted_cell():
    def __init__(self, atoms, supercell_matrix=np.eye(3)):
        self._primitive_cell = atoms
        self._supercell_matrix = supercell_matrix
        self._supercell = get_supercell(atoms, supercell_matrix, symprec=1e-4)
        self._N = np.linalg.det(supercell_matrix)

    def _get_cell_with_modulation(self, modulation):
        """
        x=x+dx
        """
        lattice = copy.deepcopy(self._supercell.get_cell())
        positions = copy.deepcopy(self._supercell.get_positions())
        masses = self._supercell.get_masses()
        symbols = self._supercell.get_chemical_symbols()
        positions += modulation.real
        scaled_positions = np.dot(positions, np.linalg.inv(lattice))
        for p in scaled_positions:
            p -= np.floor(p)
            cell = copy.deepcopy(self._supercell)
            cell.set_scaled_positions(scaled_positions)
        return cell

    def _get_displacements(self,
                           eigvec,
                           q,
                           amplitude,
                           argument,
                           mod_func=default_mod_func,
                           use_isotropy_amplitude=True):
        """
        displacements from eigvec, q, amplitude
        """
        m = self._supercell.get_masses()
        s2u_map = self._supercell.get_supercell_to_unitcell_map()
        u2u_map = self._supercell.get_unitcell_to_unitcell_map()
        s2uu_map = [u2u_map[x] for x in s2u_map]
        spos = self._supercell.get_scaled_positions()
        dim = self._supercell.get_supercell_matrix()
        r=np.dot(spos, dim.T)
        coefs = np.exp(2j * np.pi * np.dot(
            r, q)) * mod_func(r)  # Here Do not use sqrt(m)/ np.sqrt(m)
        u = []
        for i, coef in enumerate(coefs):
            eig_index = s2uu_map[i] * 3
            u.append(eigvec[eig_index:eig_index + 3] * coef)

        #u = np.array(u) / np.sqrt(len(m))
        u = np.array(u) / np.linalg.norm(u)  #/np.sqrt(self._N)
        phase_factor = self._get_phase_factor(u, argument)

        if use_isotropy_amplitude:
            amplitude = amplitude  #*self._N
        u *= phase_factor * amplitude

        return u

    def _get_phase_factor(self, modulation, argument):
        u = np.ravel(modulation)
        index_max_elem = np.argmax(abs(u))
        max_elem = u[index_max_elem]
        phase_for_zero = max_elem / abs(max_elem)
        phase_factor = np.exp(1j * np.pi * argument / 180) / phase_for_zero
        return phase_factor



def get_supercell(unitcell, supercell_matrix, symprec=1e-5):
    return Supercell(unitcell, supercell_matrix, symprec=symprec)


def get_primitive(supercell, primitive_frame, symprec=1e-5):
    return Primitive(supercell, primitive_frame, symprec=symprec)


def trim_cell(relative_axes, cell, symprec):
    """
    relative_axes: relative axes to supercell axes
    Trim positions outside relative axes

    """
    positions = cell.get_scaled_positions()
    numbers = cell.get_atomic_numbers()
    masses = cell.get_masses()
    #magmoms = cell.get_magnetic_moments()
    lattice = cell.get_cell()
    trimed_lattice = np.dot(relative_axes.T, lattice)

    trimed_positions = []
    trimed_numbers = []
    if masses is None:
        trimed_masses = None
    else:
        trimed_masses = []
    #if magmoms is None:
    #    trimed_magmoms = None
    #else:
    #    trimed_magmoms = []
    extracted_atoms = []

    positions_in_new_lattice = np.dot(positions,
                                      np.linalg.inv(relative_axes).T)
    positions_in_new_lattice -= np.floor(positions_in_new_lattice)
    trimed_positions = np.zeros_like(positions_in_new_lattice)
    num_atom = 0

    mapping_table = np.arange(len(positions), dtype='intc')
    symprec2 = symprec**2
    for i, pos in enumerate(positions_in_new_lattice):
        is_overlap = False
        if num_atom > 0:
            diff = trimed_positions[:num_atom] - pos
            diff -= np.rint(diff)
            # Older numpy doesn't support axis argument.
            # distances = np.linalg.norm(np.dot(diff, trimed_lattice), axis=1)
            # overlap_indices = np.where(distances < symprec)[0]
            distances2 = np.sum(np.dot(diff, trimed_lattice)**2, axis=1)
            overlap_indices = np.where(distances2 < symprec2)[0]
            if len(overlap_indices) > 0:
                is_overlap = True
                mapping_table[i] = extracted_atoms[overlap_indices[0]]

        if not is_overlap:
            trimed_positions[num_atom] = pos
            num_atom += 1
            trimed_numbers.append(numbers[i])
            if masses is not None:
                trimed_masses.append(masses[i])
            #if magmoms is not None:
            #    trimed_magmoms.append(magmoms[i])
            extracted_atoms.append(i)

    trimed_cell = Atoms(
        numbers=trimed_numbers,
        masses=trimed_masses,
        #magmoms=trimed_magmoms,
        scaled_positions=trimed_positions[:num_atom],
        cell=trimed_lattice, )

    return trimed_cell, extracted_atoms, mapping_table


def print_cell(cell, mapping=None, stars=None):
    symbols = cell.get_chemical_symbols()
    masses = cell.get_masses()
    magmoms = cell.get_magnetic_moments()
    lattice = cell.get_cell()
    print("Lattice vectors:")
    print("  a %20.15f %20.15f %20.15f" % tuple(lattice[0]))
    print("  b %20.15f %20.15f %20.15f" % tuple(lattice[1]))
    print("  c %20.15f %20.15f %20.15f" % tuple(lattice[2]))
    print("Atomic positions (fractional):")
    for i, v in enumerate(cell.get_scaled_positions()):
        num = " "
        if stars is not None:
            if i in stars:
                num = "*"
        num += "%d" % (i + 1)
        line = ("%5s %-2s%18.14f%18.14f%18.14f" %
                (num, symbols[i], v[0], v[1], v[2]))
        if masses is not None:
            line += " %7.3f" % masses[i]
        if magmoms is not None:
            line += "  %5.3f" % magmoms[i]
        if mapping is None:
            print(line)
        else:
            print(line + " > %d" % (mapping[i] + 1))



if __name__ == '__main__':
    test()




#/usr/bin/env python
"""
Generate domains and domain walls from phonon.
"""

from .frozen_mode import distorted_cell, default_mod_func
from pyDFTutils.ase_utils import vesta_view
from dataclasses import dataclass



class DomainBuilder():
    def __init__(self, primcell, supercell_matrix, mode_list=[]):
        """
        primitive cell: primitive cell.
        displacement mode list: [displacement vector, amplitude, argument, modulation_function]
        """
        self.primcell = primcell
        self.mode_list = mode_list
        self.supercell_matrix = supercell_matrix
        self.supercell = distorted_cell(
            self.primcell, supercell_matrix=self.supercell_matrix)

    def add_mode(self,
                 displacement,
                 qpoint,
                 amplitude,
                 argument=0,
                 modulation_function=default_mod_func):
        self.mode_list.append(
            [displacement, qpoint, amplitude, argument, modulation_function])

    def get_distortions(self):
        distortion=0.0
        for disp, q, amp, arg, mod_func in self.mode_list:
            distortion = self.supercell._get_displacements(
                self,
                disp,
                q,
                amp,
                arg,
                mod_func=mod_func,
                use_isotropy_amplitude=True)
            distortion += np.array(distortion)
        return distortion

    def get_supercell(self):
        return self.supercell._get_cell_with_modulation(self.get_distortions())

    def view_supercell(self):
        vesta_view(self.get_supercell())

    def save_supercell(self):
        pass



class DomainBuilderPhonon(DomainBuilder):
    def __init__(self, phonon, supercell_matrix, mode_list=[]):
        self.supercell=phonon.supercell

    def add_mode(self, phonon, imode, iq, amplitude, argument=0, modulation_function):
        pass



#!/usr/bin/env python
import numpy as np
from isomode.anamode import label_phbst

# Define q-points to analyze
qpoints = {
    'Gamma': [0.0, 0.0, 0.0],
    #'X': [0.5, 0.0, 0.0],
    'M': [0.5, 0.5, 0.0]
}

# Define a 2x2x2 supercell matrix
sc_matrix = np.eye(3) * 2

# Example usage of label_phbst function
label_phbst(
    fname_phbst='./LAO_PHBST.nc',  # Input PHBST.nc file
    output_fname='mode_labels.txt',            # Output file for results
    sc_mat=sc_matrix,                         # Supercell matrix
    qdict=qpoints,                            # Q-points to analyze
    tmpdir='tmp'                              # Temporary directory
)

print("Mode labeling complete. Results written to mode_labels.txt")



#!/usr/bin/env python
"""
view the amplitude of distortions with isotropy
"""
import re
import requests
import logging
import sys
import os
import tempfile
from bs4 import BeautifulSoup as BS
from collections import OrderedDict
from pathlib import Path

from ase.io import read, write
try:
    import spglib
except Exception:
    pass

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    handlers=[
        logging.StreamHandler(sys.stdout),
        logging.FileHandler('pydistort.log')
    ]
)
logger = logging.getLogger(__name__)

class PyDistortError(Exception):
    """Base exception class for pydistort errors"""
    pass

class NetworkError(PyDistortError):
    """Raised when network operations fail"""
    pass

class FileOperationError(PyDistortError):
    """Raised when file operations fail"""
    pass

def tocif(fname, outfname):
    """Convert input file to CIF format"""
    try:
        logger.info(f"Converting {fname} to CIF format: {outfname}")
        atoms = read(fname)
        atoms.set_pbc([True, True, True])
        write(outfname, atoms)
        logger.info(f"Successfully converted {fname} to CIF")
    except Exception as e:
        logger.error(f"Failed to convert {fname} to CIF: {str(e)}")
        raise FileOperationError(f"Failed to convert {fname} to CIF: {str(e)}")

def view_distort(parent_fname, distorted_fname, out_fname):
    """View the distortion between parent and distorted structures"""
    try:
        # Create temp directory
        tmpdir = tempfile.mkdtemp()
        logger.info(f"Created temporary directory: {tmpdir}")

        # Convert files to CIF
        logger.info("Converting parent file to CIF")
        parent_cif = os.path.join(tmpdir, 'parent.cif')
        tocif(parent_fname, outfname=parent_cif)

        logger.info("Converting parent CIF file to high symmetry using findsym")
        isosym = isocif(parent_cif)
        isosym.upload_cif()
        isosym.findsym()
        parent_sym_cif = os.path.join(tmpdir, 'parent_sym.cif')
        isosym.save_cif(fname=parent_sym_cif)

        logger.info("Converting distorted file to CIF")
        distorted_cif = os.path.join(tmpdir, 'distorted.cif')
        tocif(distorted_fname, outfname=distorted_cif)


        atoms = read(distorted_cif) 


        logger.info("Getting distortion mode details")
        iso = isodistort(parent_cif=parent_sym_cif, distorted_cif=distorted_cif)
        ampt = iso.get_mode_amplitude_text()
        mode_details = iso.get_mode_details(save_fname=out_fname)
        return mode_details
    except Exception as e:
        logger.error(f"Error in view_distort: {str(e)}")
        raise PyDistortError(f"Failed to analyze distortion: {str(e)}")

def view_spacegroup(filename='POSCAR', symprec=1e-3):
    """View the space group of a structure"""
    try:
        logger.info(f"Analyzing space group for {filename}")
        atoms = read(filename)
        spacegroup = spglib.get_spacegroup(atoms, symprec=symprec)
        logger.info(f"Space group determined: {spacegroup}")
        print("%20s: %s" % ('SPACEGROUP', spacegroup))
    except Exception as e:
        logger.error(f"Failed to determine space group: {str(e)}")
        raise PyDistortError(f"Failed to determine space group: {str(e)}")

class isocif(object):
    def __init__(self, fname):
        self.fname = fname
        logger.info(f"Initialized isocif with file: {fname}")

    def upload_cif(self):
        """Upload CIF file to server"""
        try:
            logger.info(f"Uploading CIF file: {self.fname}")
            data = {'input': 'uploadcif'}
            with open(self.fname, 'rb') as f:
                files = {'toProcess': (self.fname, f)}  
                ret = requests.post(
                    "https://stokes.byu.edu/iso/isocifuploadfile.php",
                    data=data,
                    files=files,
                    #allow_redirects=True
                    )
            
            text = str(ret.text)
            logger.debug(f"Upload response: {text}")

            soup = BS(text, 'lxml')
            inputs = soup.find_all('input')
            data = {i.get('name'): i.get('value') 
                   for i in inputs 
                   if i.get('type') == 'hidden' and i.get('name')}

            ret = requests.post(
                "https://stokes.byu.edu/iso/isocifform.php", data=data)
            self.upload_cif_text = ret.text
            logger.info("CIF file uploaded successfully")
        except requests.RequestException as e:
            logger.error(f"Network error during CIF upload: {str(e)}")
            raise NetworkError(f"Failed to upload CIF: {str(e)}")
        except Exception as e:
            logger.error(f"Error during CIF upload: {str(e)}")
            raise PyDistortError(f"Failed to upload CIF: {str(e)}")

    def findsym(self):
        """Find symmetry using uploaded CIF"""
        try:
            logger.info("Finding symmetry")
            soup = BS(self.upload_cif_text, 'lxml')
            inputs = soup.find_all('input')
            data = {inp.get('name'): inp.get('value') 
                   for inp in inputs 
                   if inp.get('name')}
            data["input"] = "findsym"
            
            ret = requests.post(
                "https://stokes.byu.edu/iso/isocifform.php", data=data)
            self.upload_cif_text = ret.text
            logger.info("Symmetry analysis completed")
        except requests.RequestException as e:
            logger.error(f"Network error during symmetry analysis: {str(e)}")
            raise NetworkError(f"Failed to find symmetry: {str(e)}")
        except Exception as e:
            logger.error(f"Error during symmetry analysis: {str(e)}")
            raise PyDistortError(f"Failed to find symmetry: {str(e)}")

    def save_cif(self, fname):
        """Save processed CIF to file"""
        try:
            logger.info(f"Saving CIF to {fname}")
            soup = BS(self.upload_cif_text, 'lxml')
            inputs = soup.find_all('input')
            data = {inp.get('name'): inp.get('value') 
                   for inp in inputs 
                   if inp.get('name')}
            data["input"] = "savecif"
            data["nonstandardsetting"] = 'false'
            
            ret = requests.post(
                "https://stokes.byu.edu/iso/isocifform.php", data=data)
            self.upload_cif_text = ret.text
            
            if fname is not None:
                with open(fname, 'w') as myfile:
                    myfile.write(ret.text)
                logger.info(f"CIF file saved to {fname}")
            return ret.text
        except requests.RequestException as e:
            logger.error(f"Network error while saving CIF: {str(e)}")
            raise NetworkError(f"Failed to save CIF: {str(e)}")
        except Exception as e:
            logger.error(f"Error while saving CIF: {str(e)}")
            raise PyDistortError(f"Failed to save CIF: {str(e)}")

class isodistort(object):
    def __init__(
            self,
            parent_cif="prim_sym.cif",
            distorted_cif="A_0.cif"):
        self.parent_cif = parent_cif
        self.distorted_cif = distorted_cif
        logger.info(f"Initialized isodistort with parent: {parent_cif}, distorted: {distorted_cif}")
        self.upload_parent_cif()
        self.upload_distorted_cif()
        self.select_basis()

    def upload_parent_cif(self):
        """Upload parent CIF file"""
        try:
            logger.info(f"Uploading parent CIF: {self.parent_cif}")
            data = {'input': 'uploadparentcif'}
            with open(self.parent_cif, 'rb') as f:
                files = {'toProcess': (self.parent_cif, f)} 
                ret = requests.post(
                    "https://stokes.byu.edu/iso/isodistortuploadfile.php",
                    data=data,
                    files=files,
                    #allow_redirects=True
                    )
            text = str(ret.text)
            logger.debug(f"Upload response: {text}")

            fname = re.findall(r'/tmp.*isodistort_.*.iso', text)[0]
            data = {'input': 'uploadparentcif', 'filename': fname}
            ret = requests.post(
                "https://stokes.byu.edu/iso/isodistortform.php",
                data=data,
                #allow_redirects=True
                )
            self.upload_parent_cif_text = ret.text
            logger.info("Parent CIF uploaded successfully")
        except requests.RequestException as e:
            logger.error(f"Network error uploading parent CIF: {str(e)}")
            raise NetworkError(f"Failed to upload parent CIF: {str(e)}")
        except Exception as e:
            logger.error(f"Error uploading parent CIF: {str(e)}")
            raise PyDistortError(f"Failed to upload parent CIF: {str(e)}")

    def upload_distorted_cif(self):
        """Upload distorted CIF file"""
        try:
            logger.info(f"Uploading distorted CIF: {self.distorted_cif}")
            soup = BS(self.upload_parent_cif_text, 'lxml')
            inputs = soup.find_all('input')
            data = {i.get('name'): i.get('value') 
                   for i in inputs 
                    #if i.get('name')}
                    if i.get('type') == 'hidden' and i.get('name')}

            with open(self.distorted_cif, 'rb') as f:
                files = {'toProcess': (self.distorted_cif, f)}
                ret = requests.post(
                    "https://stokes.byu.edu/iso/isodistortuploadfile.php",
                    data=data,
                    files=files,
                    #allow_redirects=True
                    )
            text = ret.text

            soup = BS(text, 'lxml')
            inputs = soup.find_all('input')
            data = {i.get('name'): i.get('value') 
                   for i in inputs 
                    #if i.get('type') == 'hidden' and i.get('name')}
                    if i.get('name')}
            data['input'] = 'uploadsubgroupcif'

            ret = requests.post(
                "https://stokes.byu.edu/iso/isodistortform.php", data=data, allow_redirects=True)
            self.upload_distorted_cif_text = ret.text
            logger.info("Distorted CIF uploaded successfully")
        except requests.RequestException as e:
            logger.error(f"Network error uploading distorted CIF: {str(e)}")
            raise NetworkError(f"Failed to upload distorted CIF: {str(e)}")
        except Exception as e:
            logger.error(f"Error uploading distorted CIF: {str(e)}")
            raise PyDistortError(f"Failed to upload distorted CIF: {str(e)}")

    def select_basis(self):
        """Select basis for distortion analysis"""
        try:
            logger.info("Selecting basis for analysis")
            soup = BS(self.upload_distorted_cif_text, 'lxml')
            inputs = soup.find_all('input')
            data = {inp.get('name'): inp.get('value') 
                   for inp in inputs 
                   if inp.get('name')}
            
            options = soup.find_all('option')
            data["inputbasis"] = "list"
            data["basisselect"] = options[1].get('value')
            data["chooseorigin"] = False
            data["trynearest"] = True
            data["domapatoms"] = 0
            data["zeromodes"] =  False
            data['input'] = 'distort'
            data['origintype'] = 'method4'

            ret = requests.post(
                "https://stokes.byu.edu/iso/isodistortform.php", data=data, allow_redirects=True)
            self.select_basis_text = ret.text
            logger.info("Basis selection completed")
        except requests.RequestException as e:
            logger.error(f"Network error during basis selection: {str(e)}")
            raise NetworkError(f"Failed to select basis: {str(e)}")
        except Exception as e:
            logger.error(f"Error during basis selection: {str(e)}")
            raise PyDistortError(f"Failed to select basis: {str(e)}")

    def get_mode_amplitude_text(self):
        """Extract mode amplitude text from analysis"""
        try:
            logger.info("Extracting mode amplitudes")
            text = self.select_basis_text
            lines = text.split('\n')
            inside = False
            amp_lines = []
            for line in lines:
                if (not inside) and line.find("ampfilename") != -1:
                    inside = True
                elif inside:
                    if line.find(r"Parameters") != -1:
                        inside = False
                    else:
                        amp_lines.append(line)
            ret = '\n'.join(amp_lines)
            logger.info("Mode amplitudes extracted successfully")
            return ret
        except Exception as e:
            logger.error(f"Error extracting mode amplitudes: {str(e)}")
            raise PyDistortError(f"Failed to extract mode amplitudes: {str(e)}")

    def get_mode_details(self, save_fname=None):
        """Get detailed mode analysis"""
        try:
            logger.info("Getting mode details")
            text = self.select_basis_text
            soup = BS(text, "lxml")
            inputs = soup.find_all('input')
            data = {inp.get('name'): inp.get('value')
                   for inp in inputs
                   #if inp.get('type') not in ['radio', 'checkbox'] and inp.get('name')}
                   if inp.get('name')}

            #options = soup.find_all('option')
            data.update({
                "topasstrain": "false",
                "treetopas": "false",
                "cifmovie": "false",
                "nonstandardsetting": 'false',
                "origintype": "modesdetails",
                "varcifmovie": 'linear',
                "zeromodes": 'false',   
                "cifdec": " 5"
            })
            ret = requests.post(
                "https://stokes.byu.edu/iso/isodistortform.php", data=data, 
                #allow_redirects=True
                )
            text = ret.text

            
            p = re.compile(r'<pre>([\s|\S]*)<\/pre>', re.MULTILINE)
            texts = p.findall(text)
            if texts:
                text = texts[0]
            else:
                text = ''
            
            soup = BS(text, "html.parser")
            text = soup.get_text()

            if save_fname is not None:
                with open(save_fname, 'w') as myfile:
                    myfile.write(text)
                logger.info(f"Mode details saved to {save_fname}")
            return text
        except requests.RequestException as e:
            logger.error(f"Network error getting mode details: {str(e)}")
            raise NetworkError(f"Failed to get mode details: {str(e)}")
        except Exception as e:
            logger.error(f"Error getting mode details: {str(e)}")
            raise PyDistortError(f"Failed to get mode details: {str(e)}")

def get_summary(text, remove0=True):
    """Get summary of mode analysis"""
    try:
        logger.info("Generating mode summary")
        modes = OrderedDict()
        for line in text.splitlines():
            if 'all' in line:
                if 'Overall' in line:
                    total = (float(line.strip().split()[1]), float(line.strip().split()[2]))
                else:
                    s = line.strip().split()
                    if remove0 is False or abs(float(s[2])) > 0.00001:
                        modes[s[0]] = (float(s[2]), float(s[3]))
        logger.info("Mode summary generated successfully")
        return total, modes
    except Exception as e:
        logger.error(f"Error generating mode summary: {str(e)}")
        raise PyDistortError(f"Failed to generate mode summary: {str(e)}")

def print_summary(text):
    """Print summary of mode analysis"""
    try:
        logger.info("Printing mode summary")
        total, modes = get_summary(text)
        for m in sorted(modes.keys(), key=lambda x: modes[x][0], reverse=True):
            v = modes[m]
            print("%20s:  %.4f  %.4f" % (m, v[0], v[1]))
        print("%20s:  %.4f  %.4f" % ('Total', total[0], total[1]))
        logger.info("Mode summary printed successfully")
    except Exception as e:
        logger.error(f"Error printing mode summary: {str(e)}")
        raise PyDistortError(f"Failed to print mode summary: {str(e)}")

def test_isocif(fname='nmodes/primitive.cif'):
    """Test isocif functionality"""
    try:
        logger.info(f"Testing isocif with file: {fname}")
        iso = isocif(fname)
        iso.upload_cif()
        iso.findsym()
        iso.save_cif(fname='save.cif')
        logger.info("isocif test completed successfully")
    except Exception as e:
        logger.error(f"Error in isocif test: {str(e)}")
        raise PyDistortError(f"isocif test failed: {str(e)}")

def test(parent_cif='save.cif', distorted_cif='nmodes/A_0.cif', mode_detail_file='mode_detail.txt'):
    """Test full distortion analysis"""
    try:
        logger.info("Running full distortion analysis test")
        iso = isodistort(parent_cif=parent_cif, distorted_cif=distorted_cif)
        ampt = iso.get_mode_amplitude_text()
        mode_details = iso.get_mode_details(save_fname=mode_detail_file)
        logger.info("Distortion analysis test completed successfully")
    except Exception as e:
        logger.error(f"Error in distortion analysis test: {str(e)}")
        raise PyDistortError(f"Distortion analysis test failed: {str(e)}")

if __name__ == "__main__":
    try:
        test_isocif()
        test()
    except Exception as e:
        logger.error(f"Error in main execution: {str(e)}")
        sys.exit(1)



import tempfile
import os
import numpy as np
from abipy.abilab import abiopen
from ase.io import read, write
from isomode.pydistort import isocif, isodistort, view_distort
from isomode.phonmode import phonon_distort_generator, displacement_cart_to_evec
from isomode.isodistort_parser import Isomode
from isomode.frozen_mode import get_supercell, distorted_cell


class LabelDDB(object):
    def __init__(self,
                 fname,
                 tmpdir=None,
                 sc_mat=np.eye(3) * 2,
                 qdict={
                     'Gamma': [0.0, 0.0, 0.0],
                     'R': [0.5, 0.5, 0.5],
                     'M':[0.5,0.5,0],
                     'X':[0,0.5,0]
                 }):
        self.fname = fname
        self.sc_mat = sc_mat
        self.qdict = qdict
        self._read_file()

        if tmpdir is None:
            self._tmpdir = tempfile.mkdtemp()
        else:
            self._tmpdir = tmpdir
            if not os.path.exists(self._tmpdir):
                os.makedirs(self._tmpdir)

    def _read_file(self):
        abifile = abiopen(self.fname)
        self.parent_atoms = abifile.structure.to_ase_atoms()
        self.parent_atoms.set_pbc([True, True, True])

        self.qpoints = abifile.phbands.qpoints.frac_coords
        self.displacements = abifile.phbands.phdispl_cart
        self.freqs=abifile.phbands.phfreqs*8056.6
        self.masses = self.parent_atoms.get_masses()

        self.idq = {}
        self.pdisps = {}
        self.edisps = {}
        self.efreqs = {}
        for name, qpt in self.qdict.items():
            for i, q in enumerate(self.qpoints):
                if np.allclose(qpt, q):
                    self.idq[name] = i
                    self.edisps[name] = self.displacements[i]
                    self.efreqs[name] = self.freqs[i]
            if name not in self.idq:
                print("The qpoint %s: %s not found in phonon" % (name, qpt))
        self.dcell = distorted_cell(
            self.parent_atoms, supercell_matrix=self.sc_mat)

    def _write_cif(self):
        """
        write structure to cif and the symmetrized cif file.
        """
        parent_cif = os.path.join(self._tmpdir, 'parent.cif')
        write(parent_cif, self.parent_atoms)
        # convert highsym_fname
        isosym = isocif(parent_cif)
        isosym.upload_cif()
        isosym.findsym()
        self.parent_sym_cif = os.path.join(self._tmpdir, 'parent_sym.cif')
        isosym.save_cif(fname=self.parent_sym_cif)

    def _write_supercell_cif(self):
        self.sc_atoms = get_supercell(
            self.parent_atoms, self.sc_mat, symprec=1e-5)

        self.sc_atoms.set_pbc([True, True, True])
        self.supercell_cif = os.path.join(self._tmpdir, 'supercell.cif')
        write(self.supercell_cif, self.sc_atoms)

    def _get_mode_details(self):
        d = isodistort(
            parent_cif=self.parent_sym_cif, distorted_cif=self.supercell_cif)
        self.mode_fname = os.path.join(self._tmpdir, 'mode_details.txt')
        d.get_mode_details(save_fname=self.mode_fname)

    def get_modes(self):
        m = Isomode(self.mode_fname)
        m.read_mode_definitions()
        #self.mode_displacements=m.get_mode_displacement()
        self.mode_definitions = m.mode_definitions

    def displ_cart_to_supercell_disp(self, displ_cart, qpt):
        evec = displacement_cart_to_evec(
            displ_cart,
            masses=self.masses/self.masses,
            scaled_positions=self.parent_atoms.get_scaled_positions(),
            qpoint=qpt,
            add_phase=True)
        disp = self.dcell._get_displacements(
            evec, qpt, amplitude=0.5, argument=0)
        return disp

    def projection(self):
        ret = {}
        masses = self.sc_atoms.get_masses()
        for qname, d in self.edisps.items():
            qpt = self.qdict[qname]
            for imode, mode in enumerate(d):
                freq=self.efreqs[qname][imode]
                sc_disp = self.displ_cart_to_supercell_disp(mode,
                                                            qpt).flatten()
                #sc_evec = np.array(sc_disp) * np.sqrt(
                #    np.kron(masses, [1, 1, 1]))
                sc_evec = np.real(sc_disp) / np.linalg.norm(sc_disp)
                print("================ imode: %s , freqs: %s ========== "%(imode,freq ) )
                #print(np.real(sc_disp).reshape((40,3)))
                #print(np.real(mode))
                for fullname, md in self.mode_definitions.items():
                    mdkpt = md['mode_info']['kpt']
                    mdlabel = md['mode_info']['label']
                    mdsym =md['mode_info']['symmetry']
                    mddisp = md['mode'].flatten()

                    #mdevec = np.array(mddisp) * np.sqrt(
                    #    np.kron(masses, [1, 1, 1]))
                    mdevec=mddisp
                    mdevec = np.real(mdevec) / np.linalg.norm(mdevec)

                    proj = np.abs(np.dot(sc_evec, mdevec))
                    #if np.allclose(mdkpt, qpt) and proj > 0.5:
                    if proj>0.5:
                        #print("md_evec", mdevec)
                        #print("sc_evec", sc_evec)
                        print(fullname)
                        #print(mdevec.reshape((len(self.sc_atoms), 3)))
                        print(mdlabel)
                        print(proj)
                        ret[(qname, imode)] = {'freq': freq, 'label': mdlabel,  'symmetry': mdsym}
        self.mode_info=ret
        return ret

    def get_labels(self):
        self._write_cif()
        self._write_supercell_cif()
        self._get_mode_details()
        self.get_modes()
        return self.projection()

    def write_result(self, fname):
        self.get_labels()
        with open(fname, 'w') as myfile:
            myfile.write("# %7s  %5s  %10s  %5s  %9s\n"%("qname",'id', 'freq(cm-1)', 'label','symmetry'))
            for key, val in self.mode_info.items():
                qname, imode =key
                freq=val['freq']
                label=val['label']
                sym=val['symmetry']
                myfile.write("%8s  %5d  %10.5e  %5s  %9s\n"%(qname, imode, freq, label, sym))


def test():
    l = LabelDDB('./phbst/run.abo_PHBST.nc', tmpdir='./tmp')
    l.write_result('result.txt')


test()



import os
import numpy as np
from scipy.optimize import minimize_scalar
from abipy.abilab import abiopen
from isomode.frozen_mode import distorted_cell
from ase import Atoms
from ase.io import write
import spglib.spglib
import matplotlib.pyplot as plt
from functools import partial
from numpy import array
import pickle


def displacement_cart_to_evec(displ_cart,
                              masses,
                              scaled_positions,
                              qpoint=None,
                              add_phase=True):
    """
    displ_cart: cartisien displacement. (atom1_x, atom1_y, atom1_z, atom2_x, ...)
    masses: masses of atoms.
    scaled_postions: scaled postions of atoms.
    qpoint: if phase needs to be added, qpoint must be given.
    add_phase: whether to add phase to the eigenvectors.
    """
    if add_phase and qpoint is None:
        raise ValueError('qpoint must be given if adding phase is needed')
    m = np.sqrt(np.kron(masses, [1, 1, 1]))
    evec = displ_cart * m
    if add_phase:
        phase = [
            np.exp(-2j * np.pi * np.dot(pos, qpoint))
            for pos in scaled_positions
        ]
        phase = np.kron(phase, [1, 1, 1])
        evec *= phase
        evec /= np.linalg.norm(evec)
    return evec


def rotate_evecs(evecs, theta):
    length = len(evecs)
    if length == 3:
        print(length)
    if length != 2:
        return evecs
        #raise NotImplementedError("Only multiplicity=2 rotation implemented. Here multiplicity=%s"%length)
    #R=np.array([[np.cos(theta), -np.sin(theta) ],
    #            [np.sin(theta), np.cos(theta)]])
    #newevec=np.copy(evec)
    #for i in range(length):
    #    newevec[0+i*3:2+i*3]=np.dot( evec[0+i*3:2+i*3],R)
    u = np.array(evecs[0])
    v = np.array(evecs[1])
    newvec1 = np.cos(theta) * u - np.sin(theta) * v
    newvec2 = np.sin(theta) * u + np.cos(theta) * v
    return [newvec1, newvec2]


def project_to_x(vec):
    xs = vec[::3]
    ys = vec[1::3]
    return np.linalg.norm(xs, ord=1) - np.linalg.norm(ys, ord=1)


def test_align(a):
    thetas = np.arange(0, np.pi, 0.01)
    rxs = []
    for theta in thetas:
        #print(theta)
        rx = rotate_evecs(a, theta)[0]
        #print(rx)
        #print(project_to_x(rx))
        rxs.append(project_to_x(rx))
    plt.plot(thetas, rxs)
    plt.show()


rfunc = lambda theta, modes: -project_to_x(rotate_evecs(modes, theta)[0])


def align_degenerate_modes(modes):
    """
    modes are degenerate
    """
    #modes=np.real(modes)
    nmodes = len(modes)
    func = partial(rfunc, modes=modes)
    res = minimize_scalar(func, tol=1e-19)  #, bounds=[0, np.pi])
    theta = res.x
    new_modes = rotate_evecs(modes, theta)
    #print("theta: ", theta / np.pi)
    for i in range(len(modes)):
        mi = modes[i] / np.linalg.norm(modes[i])
        nmi = new_modes[i] / np.linalg.norm(new_modes[i])
        #print("Aligning=========\n", np.real(mi).reshape((16,3)), np.real(nmi).reshape((16,3)))
    return new_modes


def align_all_modes(evals, evecs, tol=1e-7):
    """
    Here we assum evals are already sorted.
    """
    multi_modes = []
    multi_modes_inds = []
    new_evals = []
    new_evecs = []
    for i, evec in enumerate(evecs):
        # each time evec[i]!=evec[i-1], deal with mutli_modes and empty it,
        # then push eve[i] into multi_modes
        if np.abs(evals[i - 1] - evals[i]) > tol:
            if len(multi_modes) == 1:  # multi=1, no need to align
                new_evecs.append(multi_modes[0])
            elif len(multi_modes) > 1:  # multi, align and save
                for e in align_degenerate_modes(multi_modes):
                    new_evecs.append(e)
                if len(multi_modes) == 3:  # multi, align and save
                    print(evals[i - 1], evec)
            # clean multi_modes and save new
            multi_modes = []
            multi_modes.append(evec)
        else:
            multi_modes.append(evec)
    return new_evecs


class phonon_distort_generator(object):
    def __init__(self, fname, qdict):
        self.fname = fname
        self.qdict = qdict
        self.read_phbst_file()
        self.distorted_structures = []

    def read_phbst_file(self):
        self.phbst = abiopen(self.fname)
        self.atoms = self.phbst.structure.to_ase_atoms()
        # write('primitive.cif',atoms)
        self.nbranch = 3 * len(self.atoms)
        self.masses = self.atoms.get_masses()
        self.scaled_positions = self.atoms.get_scaled_positions()

    def generate_primitive_structure(self, path=None):
        self.atoms.set_pbc([1, 1, 1])
        if path is not None:
            if not os.path.exists(path):
                os.makedirs(path)
            write(os.path.join(path, 'primitive.cif'), self.atoms)
        return self.atoms

    def generate_combined_mode_distorted_structures(
            self,
            supercell_matrix=np.eye(3) * 2,
            amplitude=1.0,
            max_freq=0.0,
            path=None,
            align_evecs=False,
            align_disp=False, ):
        """
        qname, qpoint, ibranch, freq, structure, spacegroup
        """
        if not os.path.exists(path):
            os.makedirs(path)

        disp=0.0
        for qname, qpt in self.qdict.items():
            displ_carts = [
                self.phbst.get_phmode(qpt, i).displ_cart
                for i in range(self.nbranch)
            ]
            freqs = [
                self.phbst.get_phmode(qpt, i).freq
                for i in range(self.nbranch)
            ]
            if align_evecs:
                evecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in displ_carts
                ]

                nevecs = align_all_modes(freqs, evecs)
            elif align_disp:
                ndispl_carts = align_all_modes(freqs, displ_carts)
                nevecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in ndispl_carts
                ]
            else:
                ndispl_carts = displ_carts
                nevecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in ndispl_carts
                ]
            scell = distorted_cell(
                        self.atoms, supercell_matrix=supercell_matrix)
            for i in range(self.nbranch):
                freq = freqs[i]
                if freq < max_freq:
                    evec = nevecs[i]
                    scell = distorted_cell(
                        self.atoms, supercell_matrix=supercell_matrix)
                    disp += scell._get_displacements(
                        evec, qpt, amplitude=amplitude, argument=0)
            newcell = scell._get_cell_with_modulation(disp)
            newcell = Atoms(newcell)
            spacegroup = spglib.get_spacegroup(newcell, symprec=1e-3)
            if path is not None:
                newcell.set_pbc([1, 1, 1])
                write(os.path.join(path, 'combined_modes.cif'), newcell)

    def generate_distorted_structures(self,
                                      supercell_matrix=np.eye(3) * 2,
                                      amplitude=1.0,
                                      unstable_only=True,
                                      path=None):
        """
        qname, qpoint, ibranch, freq, structure, spacegroup
        """
        if not os.path.exists(path):
            os.makedirs(path)
        for qname, qpt in self.qdict.items():
            displ_carts = [
                self.phbst.get_phmode(qpt, i).displ_cart
                for i in range(self.nbranch)
            ]
            freqs = [
                self.phbst.get_phmode(qpt, i).freq
                for i in range(self.nbranch)
            ]
            align_evecs = False
            align_disp = False
            if align_evecs:
                evecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in displ_carts
                ]

                nevecs = align_all_modes(freqs, evecs)
            elif align_disp:
                ndispl_carts = align_all_modes(freqs, displ_carts)
                nevecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in ndispl_carts
                ]
            else:
                ndispl_carts = displ_carts
                nevecs = [
                    displacement_cart_to_evec(
                        displ_cart,
                        masses=self.masses,
                        scaled_positions=self.scaled_positions,
                        qpoint=qpt,
                        add_phase=True) for displ_cart in ndispl_carts
                ]

            for i in range(self.nbranch):
                freq = freqs[i]
                if freq < 0 or (not unstable_only):
                    print(len(nevecs))
                    evec = displ_carts[i]#nevecs[i]
                    print(evec)
                    scell = distorted_cell(
                        self.atoms, supercell_matrix=supercell_matrix)
                    disp = scell._get_displacements(
                        evec, qpt, amplitude=amplitude, argument=0)
                    newcell = scell._get_cell_with_modulation(disp)
                    newcell = Atoms(newcell)
                    spacegroup = spglib.get_spacegroup(newcell, symprec=1e-3)
                    if path is not None:
                        newcell.set_pbc([1, 1, 1])
                        write(
                            os.path.join(path, '%s_%s.cif' % (qname, i)),
                            newcell)
                    self.distorted_structures.append({
                        'qname': qname,
                        'qpoint': qpt,
                        'ibranch': i,
                        'evec': evec,
                        'amplitude': amplitude,
                        'spacegroup': spacegroup,
                        'atoms': newcell
                    })
        if path is not None:
            pfname = os.path.join(path, 'phmodes.pickle')
            with open(pfname, 'wb') as myfile:
                pickle.dump(self.distorted_structures, myfile)
        return self.distorted_structures


def test_phbst_modes(fname='../data/DionJ/phonon/run.abo_PHBST.nc'):
    qdict = {
        'Gamma': [0.0, 0.0, 0.0],
        'Xy': [0, 0.5, 0],
        #'Xx': [0.5, 0.0, 0],
        'M': [0.5, 0.5, 0],
        'Rx': [0.5, 0.0, 0.5],
        #'Ry': [0.0, 0.5, 0.5],
        'A': [0.5, 0.5, 0.5],
        'Z': [0, 0, 0.5]
    }
    dg = phonon_distort_generator(fname, qdict)
    dg.generate_primitive_structure(path='nmodes')
    ds = dg.generate_distorted_structures(
        supercell_matrix=np.eye(3) * 2,
        amplitude=0.4,
        unstable_only=True,
        path='nmodes')
    print(ds)


#test_phbst_modes()



import numpy as np
import os
import spglib
from ase import Atoms
from ase.io import read, write
from pyDFTutils.ase_utils import symbol_number, symnum_to_sym, vesta_view
from collections import OrderedDict, defaultdict
import re
from itertools import combinations
import copy
import json
import pickle
from itertools import combinations


def parse_mode_name_line(line=None):
    """
    return a dictionary ret
    ret['kpt']: the kpoint. a list. Note: In python2, 1/2 -> 0 . So it could be wrong in python2.
    ret['kpt_string']: kpt in string. 1/2 is still 1/2
    ret['normfactor']
    ret['label']: eg GM3+
    ret['symmetry']: eg A2u
    fullname:  '[0,0,0]GM1+[Nb2:g:dsp]A1(a)'
    direction: a/b/c
    """
    kpt_string = re.findall(r'\[.*\d\]', line)[0]
    kpt = (eval(kpt_string))
    normfactor_string = re.findall(r'normfactor\s*=\s*(.*)', line)[0]
    normfactor = float(normfactor_string)
    label, symmetry = re.findall(r'\](.*?)\(', line)

    a1 = re.findall(r'(\[.*?)\(', line)[0]
    a2 = re.findall(r'\[.*?\)', line)[1]
    fullname = a1 + a2
    direction = fullname[-2]

    return {
        'kpt': kpt,
        'kpt_string': kpt_string,
        'normfactor': normfactor,
        'label': label,
        'symmetry': symmetry,
        'direction': direction,
        'fullname': fullname,
    }


def parse_mode_amp_line(line):
    """
    return dict, of which the keys are
    kpt
    kpt_string
    label
    symmetry
    direction
    fullname
    amp
    """
    fullname, amp, _, _ = line.strip().split()
    amp = float(amp)
    kpt_string = re.findall(r'\[.*\d\]', line)[0]
    kpt = (eval(kpt_string))
    label = re.findall(r'\](.*?)\[', line)[0]
    symmetry = re.findall(r'.*\](.*?)\(',
                          line)[0]  # first .* is greedy to eat all before ]
    direction = fullname[-2]

    return {
        'kpt': kpt,
        'kpt_string': kpt_string,
        'label': label,
        'symmetry': symmetry,
        'direction': direction,
        'fullname': fullname,
        'amp': amp
    }


#print(parse_mode_amp_line(
#    "[0,0,0]GM5-[Cs1:d:dsp]Eu(a)         0.00000   0.00000  0.00000"))


def find_primitive(atoms, symprec=1e-4):
    """
    find the primitive cell withh regard to the magnetic structure. a atoms object is returned.
    """
    #atoms_mag,sym_dict=ref_atoms_mag(atoms)
    cell, scaled_pos, chem_nums = spglib.find_primitive(atoms, symprec=symprec)
    #print(spglib.get_symmetry(atoms, symprec=1e-5))
    chem_sym = 'H%d' % (len(chem_nums))
    new_atoms = Atoms(chem_sym)

    new_atoms.set_atomic_numbers(chem_nums)
    new_atoms.set_cell(cell)
    new_atoms.set_scaled_positions(scaled_pos)
    #new_atoms=rev_ref_atoms(new_atoms,sym_dict)
    return new_atoms


#print(
#    parse_mode_name_line(
#        line="P4/mmm[0,0,0]GM1+(a)[Nb2:g:dsp]A1(a) normfactor = 0.00825"))


def split_symnum(symnum):
    """
    symnum-> sym. eg: Fe1-> Fe
    """
    try:
        a = re.search('[A-Za-z]+', symnum).group()
        b = int(symnum[len(a):])
        return a, b
    except AttributeError:
        raise AttributeError('%s is not a good symbol_number' % symnum)


def read_mode_amplitudes(filename='mtable.org',
                         use_direction=True,
                         randomize=False,
                         use_abs=False,
                         use_max=False):
    """
    detail_mode_amplitude_dict:  {fullname: amp}
    total_mode_dict: {shortname: amp} Note: sqrt(sum(amp_detail**2))
    total_mode_definitions: shortname: {fullname: amp}. Normalized to norm(amp_shortname)
    """
    detail_mode_amplitude_dict = defaultdict(float)
    total_mode_dict = defaultdict(float)
    total_mode_definitions = dict()
    with open(filename) as myfile:
        lines = myfile.readlines()
    for line in lines:
        try:
            result = parse_mode_amp_line(line)
        except Exception:
            result = None
        if result is not None:
            fullname = result['fullname']
            label = result['label']
            direction = result['direction']
            if use_direction:
                shortname = (label, direction)
            else:
                shortname = label

            amp = result['amp']
            if use_abs:
                amp = abs(amp)
            if np.abs(amp) > 0.01:
                if randomize:
                    amp = amp * (1.0 + (random.random() - 1.0) * 0.03)
                detail_mode_amplitude_dict[fullname] += amp
                total_mode_dict[
                    shortname] += amp**2  # **2 because amp= sqrt(sum(amp_i**2))
                if shortname not in total_mode_definitions:
                    total_mode_definitions[shortname] = defaultdict(float)
                    total_mode_definitions[shortname][fullname] += amp
                else:
                    total_mode_definitions[shortname][fullname] += amp
            #print(m['label'], m['direction'], m['amp'])
    if use_max:
        total_mode_definitions_max = dict()
        for key, val in total_mode_definitions.items():
            #print(max(val.items(), key=lambda x: x[1]))
            total_mode_definitions_max[key] = dict(
                [max(val.items(), key=lambda x: abs(x[1]))])
        total_mode_definitions = total_mode_definitions_max

    # normalization:
    for shortname in total_mode_dict:
        total_mode_dict[shortname] = np.sqrt(total_mode_dict[shortname])
        for fullname in total_mode_definitions[shortname]:
            total_mode_definitions[shortname][
                fullname] = total_mode_definitions[shortname][
                    fullname] / total_mode_dict[shortname]

    return detail_mode_amplitude_dict, total_mode_dict, total_mode_definitions


def test_read_mode_amplitudes():
    print(read_mode_amplitudes('mtable.org', use_abs=True, use_max=True)[2])


#print(read_mode_amplitudes('mtable.org', use_direction=False, use_abs=True, use_max=False)[2])


class Isomode(object):
    def __init__(self, fname):
        self.fname = fname
        with open(self.fname) as myfile:
            self.lines = myfile.readlines()
        self.atoms = None
        self.cellpars = []
        self.symbols = []
        self.positions = []
        self.symdict = {}
        self.natom = None
        self.mode_definitions = {}
        self.read_supercell()
        self.read_mode_definitions()

    def read_primitive_cell(self):
        pass

    def read_supercell(self):
        inside = False
        sympos = []
        iatom = 0
        for iline, line in enumerate(self.lines):
            if line.strip().startswith('Undistorted superstructure'):
                inside = True
                continue
            if inside:
                if line.strip().startswith('a='):
                    # eg. a=7.80962, b=7.80962, c=30.28940, alpha=90.00000, beta=90.00000, gamma=90.00000
                    segs = line.strip().split(',')
                    for seg in segs:
                        self.cellpars.append(float(seg.strip().split('=')[1]))
                elif line.strip().startswith('atom'):
                    pass
                elif line.strip() == '':
                    inside = False
                else:
                    symnum, site, x, y, z, occ, displ = line.strip().split()
                    sym, num = split_symnum(symnum)
                    x = float(x)
                    y = float(y)
                    z = float(z)
                    occ = float(occ)
                    displ = float(displ)
                    sympos.append([symnum, x, y, z])
                    self.symbols.append(sym)
                    self.positions.append(np.array([x, y, z]))
                    self.symdict[symnum] = iatom
                    iatom = iatom + 1

        self.atoms = Atoms(
            symbols=self.symbols,
            scaled_positions=self.positions,
            cell=self.cellpars)
        self.natom = len(self.atoms)
        self.cell = self.atoms.get_cell()

    def set_supercell_cellpar(self, cell, supercell_matrix=np.eye(3)):
        self.cell = np.dot(cell, supercell_matrix)
        self.atoms = Atoms(
            symbols=self.symbols,
            scaled_positions=self.positions,
            cell=self.cellpars)
        self.cellpar = self.atoms.get_cell_lengths_and_angles()

    def substitute_element(self, subs_dict):
        symbols = self.symbols
        new_symbols = []
        for sym in symbols:
            if sym in subs_dict:
                new_symbols.append(subs_dict[sym])
            else:
                new_symbols.append(sym)
        self.symbols = new_symbols
        self.atoms = Atoms(
            symbols=self.symbols,
            scaled_positions=self.positions,
            cell=self.cellpars)

    def read_mode_definitions(self):
        """
        read displacive mode definitions
        """
        inside = False
        mode_definitions = OrderedDict()
        inside_mode = False
        for iline, line in enumerate(self.lines):
            if line.strip().startswith("Displacive mode definitions"):
                inside = True
                continue
            elif line.strip().startswith("Displacive mode amplitudes"):
                inside = False
            elif inside:
                if line.find('normfactor') != -1:  # begining of one mode
                    nameline = line
                    r = parse_mode_name_line(line=nameline)
                    mode_info = {}
                    for key in r:
                        mode_info[key] = r[key]
                    kpt = r['kpt']
                    kpt_string = r['kpt_string']
                    normfactor = r['normfactor']
                    label = r['label']
                    symmetry = r['symmetry']
                    fullname = r['fullname']
                    inside_mode = True
                    #deltas = {}
                    mode = np.zeros([self.natom, 3], dtype=float)
                    continue
                if inside_mode:
                    if line.strip() == '':  # end of one mode.
                        inside_mode = False
                        self.mode_definitions[mode_info['fullname']] = {
                            'mode_info': mode_info,
                            'mode': mode
                        }
                    elif line.strip().startswith('atom'):
                        pass
                    else:  # a line  of displacement.
                        symnum, x, y, z, dx, dy, dz = line.strip().split()
                        #sym, num = split_symnum(symnum)
                        _, _, _, dx, dy, dz = map(float, (x, y, z, dx, dy, dz))
                        delta = (dx, dy, dz)
                        #deltas[symnum] = delta
                        mode[self.symdict[symnum]] = delta

    def get_mode_info(self, fullname, info_key):
        """
        return information of a mode by fullname
        fullname: eg.  [0,0,0]GM5-[Cs1:d:dsp]Eu(a)
        info_key: kpt|kpt_string|norm_factor|label|symmetry|fullanme|delta
        """
        if (info_key == 'displacement') or (info_key == 'mode'):
            return self.mode_definitions[fullname]['mode']
        else:
            return self.mode_definitions[fullname]['mode_info'][info_key]

    def get_mode_displacement(self, fullname):
        return self.mode_definitions[fullname]['mode']

    def get_all_mode_fullname(self):
        return self.mode_definitions.keys()

    def get_undistorted_structure(self):
        return self.atoms

    def get_total_mode_displacement(self):
        pass

    def get_total_mode_info(self):
        pass

    def get_distorted_structure(self, mode_dict, primitive=True, amp_multiplier=1.0):
        distorted_structure = self.atoms.copy()
        scaled_positions = distorted_structure.get_scaled_positions()
        for fullname in mode_dict:
            amp = mode_dict[fullname]*amp_multiplier
            normfactor = self.mode_definitions[fullname]['mode_info'][
                'normfactor']
            disp = [
                normfactor * amp * np.dot(self.atoms.get_cell(), m)
                for m in self.mode_definitions[fullname]['mode']
            ]
            scaled_positions += disp

        distorted_structure.set_scaled_positions(scaled_positions)
        #print(spglib.get_spacegroup(distorted_structure, symprec=1e-4))
        if primitive:
            distorted_structure = find_primitive(
                distorted_structure, symprec=1e-4)
            #print(distorted_structure)
        return distorted_structure

    def read_mode_amplitudes(self, fname=None, use_direction=True, **kwargs):
        """
        it read amplitudes from a file.
        saved information:
        1. each detailed mode amplitude
        2. each total mode (defined by label+direction) amplitude
        3. a map to total_mode from detailed mode.
        """
        if fname is None:
            fname = self.fname
        (self.detail_mode_amplitude_dict, self.total_mode_dict,
         self.total_mode_definitions) = read_mode_amplitudes(
             fname, use_max=False)
        #print(self.total_mode_definitions)
        #print(self.total_mode_dict)

    def get_total_modes(self):
        return self.total_mode_dict

    def prepare_structure_single(self,
                                 pickle_file_name=None,
                                 amp=None,
                                 cif_dir=None,
                                 primitive=False):
        #print(self.total_mode_definitions)
        structure_list = []
        if cif_dir is not None:
            if not os.path.exists(cif_dir):
                os.makedirs(cif_dir)
        for key, mode_dict in self.total_mode_definitions.items():
            label, direction = key
            # Note sometimes 'b' without 'a'
            if (direction == 'a') or (
                (label, 'a') not in self.total_mode_definitions):
                if amp is not None:
                    for fullname in mode_dict:
                        mode_dict[fullname] *= amp
                distorted_structure = self.get_distorted_structure(
                    mode_dict, primitive=primitive)
                #print(distorted_structure)
                spgroup = spglib.get_spacegroup(
                    distorted_structure, symprec=1e-4)
                filename = '%s_%s.cif' % (label, direction)
                print('%s %s %s %s\n' % (label, direction, spgroup, filename))
                structure_list.append(
                    [label, direction, spgroup, filename, distorted_structure])
                if cif_dir is not None:
                    cif_fname = os.path.join(cif_dir, filename)
                    write(cif_fname, distorted_structure)
        if pickle_file_name is not None:
            #pickle_file = open(pickle_file_name, 'w')
            #pickle_list = tuple(item[:-1] for item in structure_list)
            with open(pickle_file_name, 'wb') as pickle_file:
                pickle.dump(structure_list, pickle_file)
        self.structure_list_a = structure_list
        return structure_list

    def prepare_structure_ab(self,
                             pickle_file_name=None,
                             amp=None,
                             cif_dir=None,
                             primitive=False):
        #print(self.total_mode_definitions)
        structure_list = []
        if cif_dir is not None:
            if not os.path.exists(cif_dir):
                os.makedirs(cif_dir)
        for key1, mode_dict1 in self.total_mode_definitions.items():
            for key2, mode_dict2 in self.total_mode_definitions.items():

                label1, direction1 = key1
                label2, direction2 = key2
                # Note sometimes 'b' without 'a'
                if (direction1 == 'a') and (direction2 == 'b'):
                    mode_dict = defaultdict(float)
                    for mdict in [mode_dict1, mode_dict2]:
                        mode_dict.update(mdict)
                    distorted_structure = self.get_distorted_structure(
                        mode_dict, primitive=primitive)
                    #print(distorted_structure)
                    spgroup = spglib.get_spacegroup(
                        distorted_structure, symprec=1e-4)
                    name = '%s_%s_%s_%s.cif' % (label1, direction1, label2,
                                                direction2)
                    filename = name + '.cif'
                    structure_list.append(
                        [name, spgroup, filename, distorted_structure])
                    if cif_dir is not None:
                        cif_fname = os.path.join(cif_dir, filename)
                        write(cif_fname, distorted_structure)
        if pickle_file_name is not None:
            #pickle_file = open(pickle_file_name, 'w')
            #pickle_list = tuple(item[:-1] for item in structure_list)
            with open(pickle_file_name, 'wb') as pickle_file:
                pickle.dump(structure_list, pickle_file)
        self.structure_list_ab = structure_list
        return structure_list

    def generate_distorted_structure_from_modelist(self, modelist, amp=0.2, primitive=False):
        """
        modelist: eg. (('X3+','a'), ('GM5-', 'b'))
        """
        #for key1, mode_dict1 in self.total_mode_definitions.items():
        #    for key2, mode_dict2 in self.total_mode_definitions.items():

        #        label1, direction1 = key1
        #        label2, direction2 = key2
                # Note sometimes 'b' without 'a'
        mode_dict = defaultdict(float)
        for m in modelist:
            for key in m:
                mode_detail=self.total_mode_definitions[m]
                #for mdetail in mode_detail:
                mode_dict.update(mode_detail)
        print(mode_dict)
        
        distorted_structure = self.get_distorted_structure(
                        mode_dict, primitive=primitive, amp_multiplier=amp)
        return distorted_structure

    def prepare_structure(self, pickle_fname='all_modes01.pickle', cif_dir='all_modes0.03', amp=0.03, primitive=True):
        single_modes = sorted(self.get_total_modes().keys())
        Xa = Xa_modes(single_modes)
        Xab = Xab_modes(single_modes)
        XaYb = XaYb_modes(single_modes)
        XabYa = XabYa_modes(single_modes)
        XabYab = XabYab_modes(single_modes)
        ret=OrderedDict()
        for ms in [Xa, Xab, XaYb, XabYa, XabYab]:
            for m in ms:
                print(m)
                name=name_modes(m)
                atoms=self.generate_distorted_structure_from_modelist(m, amp=amp, primitive=primitive)
                spgroup = spglib.get_spacegroup(atoms, symprec=1e-4)
                #myfile.write('|%s|%s|%s|%s|\n' % (label, n_direction, spgroup, name))
                ret[name]=atoms
                if cif_dir is not None:
                    if not os.path.exists(cif_dir):
                        os.makedirs(cif_dir)
                    fname=os.path.join(cif_dir, name+'.cif')
                    atoms.set_pbc([True, True, True])
                    write(fname, atoms)
        # print(ret)
        if pickle_fname is not None:
            with open(pickle_fname,'wb') as myfile:
                pickle.dump(ret, myfile)


def read_unstable_modes(fname='./mtable.org'):
    with open(fname) as myfile:
        lines = myfile.readlines()
    for line in lines:
        if line.strip().startswith('[') and not line.find('all') != -1:
            m = parse_mode_amp_line(line)


modes = {
    '[0,0,1/2]Z5-[Ca1:h:dsp]E(a)': 1.0,
    '[0,0,1/2]Z5-[Ca1:h:dsp]E(b)': 1.0,
    '[0,0,0]GM5+[Ca1:h:dsp]E(a)': 1.0,
    '[0,0,0]GM5+[Ca1:h:dsp]E(b)': 1.0,
    '[0,0,0]GM5-[Cs1:d:dsp]Eu(a)': 1.0,
    '[0,0,0]GM5-[Cs1:d:dsp]Eu(b)': 1.0,
    '[1/2,1/2,1/2]A1-[O5:i:dsp]B1(a)': 1.0,
    '[1/2,1/2,1/2]A2+[O1:f:dsp]B2u(a)': 1.0,
    '[1/2,1/2,1/2]A3+[O1:f:dsp]B3u(a)': 1.0,
    '[1/2,1/2,0]M3+[O1:f:dsp]B3u(a)': 1.0,
    '[1/2,1/2,0]M5+[O1:f:dsp]B1u(a)': 1.0,
    '[0,1/2,0]X3+[O3:g:dsp]E(a)': 1.0,
    '[0,1/2,0]X3+[O3:g:dsp]E(b)': 1.0,
    '[0,1/2,1/2]R3+[O3:g:dsp]E(a)': 1.0,
    '[0,1/2,1/2]R3+[O3:g:dsp]E(b)': 1.0,
}


def label_direction_modes(modes):
    ldmodes = {}
    for mode in modes:
        fullname = mode
        label = re.findall(r'\](.*?)\[', fullname)[0]
        symmetry = re.findall(
            r'.*\](.*?)\(',
            fullname)[0]  # first .* is greedy to eat all before ]
        direction = fullname[-2]
        ldmode = (label, direction)
        #ldmodes.append(ldmode)
        ldmodes[ldmode] = fullname

    return ldmodes


def group_direction(modes, direction='a'):
    ldmodes = label_direction_modes(modes)
    return dict([(x, ldmodes[x]) for x in ldmodes if x[1] == direction])


def test():
    myparser = isodistort_parser('./isodistort_modes.txt')
    #print(myparser.modes)
    #vesta_view(myparser.get_distorted_structure({'[0,0,1/2]Z5-[O9:g:dsp]E(a)': 1.0}))
    #vesta_view(

    mode_groups = dict(
        Z5_m_1={
            '[0,0,1/2]Z5-[Ca1:h:dsp]E(a)': 1.0,
        },
        Z5_m_2={
            '[0,0,1/2]Z5-[Ca1:h:dsp]E(a)': 1.0,
            '[0,0,1/2]Z5-[Ca1:h:dsp]E(b)': 1.0,
        },
        GM5_p_1={
            '[0,0,0]GM5+[Ca1:h:dsp]E(a)': 1.0,
        },
        GM5_p_2={
            '[0,0,0]GM5+[Ca1:h:dsp]E(a)': 1.0,
            '[0,0,0]GM5+[Ca1:h:dsp]E(b)': 1.0
        },
        GM5_m_1={
            '[0,0,0]GM5-[Cs1:d:dsp]Eu(a)': 1.0,
        },
        GM5_m_2={
            '[0,0,0]GM5-[Cs1:d:dsp]Eu(a)': 1.0,
            '[0,0,0]GM5-[Cs1:d:dsp]Eu(b)': 1.0,
        },
        A1_m_1={'[1/2,1/2,1/2]A1-[O5:i:dsp]B1(a)': 1.0},
        A2_p_1={
            '[1/2,1/2,1/2]A2+[O1:f:dsp]B2u(a)': 1.0,
        },
        A3_p_1={
            '[1/2,1/2,1/2]A3+[O1:f:dsp]B3u(a)': 1.0,
        },
        M3_p_1={
            '[1/2,1/2,0]M3+[O1:f:dsp]B3u(a)': 1.0,
        },
        M5_p_1={
            '[1/2,1/2,0]M5+[O1:f:dsp]B1u(a)': 1.0,
        },
        X3_p_1={
            '[0,1/2,0]X3+[O3:g:dsp]E(a)': 1.0,
        },
        X3_p_2={
            '[0,1/2,0]X3+[O3:g:dsp]E(a)': 1.0,
            '[0,1/2,0]X3+[O3:g:dsp]E(b)': 1.0,
        },
        R3_p_1={
            '[0,1/2,1/2]R3+[O3:g:dsp]E(a)': 1.0,
        },
        R3_p_2={
            '[0,1/2,1/2]R3+[O3:g:dsp]E(a)': 1.0,
            '[0,1/2,1/2]R3+[O3:g:dsp]E(b)': 1.0,
        })
    myfile = open('single_mode.org', 'w')
    myfile.write('|label|ndirection|spacegroup|\n')
    for name, modes in mode_groups.items():
        label = myparser.modes[list(modes.keys())[0]]['mode_info']['label']
        n_direction = len(modes)
        distorted_structure = myparser.get_distorted_structure(modes)

        spgroup = spglib.get_spacegroup(distorted_structure, symprec=1e-4)
        myfile.write('|%s|%s|%s|%s|\n' % (label, n_direction, spgroup, name))
    myfile.close()

    myfile = open('double_mode.org', 'w')
    myfile.write('|label1|ndirection1|label2|ndirection2|spacegroup|\n')
    for comb in combinations(mode_groups.items(), 2):
        g1, g2 = comb
        #print(g1)
        key, modes1 = g1
        label1 = myparser.modes[list(modes1.keys())[0]]['mode_info']['label']
        n_direction1 = len(modes1)

        key, modes2 = g2
        label2 = myparser.modes[list(modes2.keys())[0]]['mode_info']['label']
        n_direction2 = len(modes2)

        if label1 == label2:
            continue
        else:
            modes = copy.deepcopy(modes1)
            modes.update(modes2)
            distorted_structure = myparser.get_distorted_structure(modes)
            spgroup = spglib.get_spacegroup(distorted_structure, symprec=1e-4)
            myfile.write('|%s|%s|%s|%s|%s|\n' % (label1, n_direction1, label2,
                                                 n_direction2, spgroup))
    myfile.close()

    myparser.get_distorted_structure({
        '[0,0,1/2]Z5-[Ca1:h:dsp]E(a)': 1.0,
        #'[0,0,1/2]Z5-[Ca1:h:dsp]E(b)':1.0,
        '[0,0,1/2]Z5-[O1:f:dsp]B3u(a)': 1.0,
        #'[0,0,1/2]Z5-[O1:f:dsp]B3u(b)':1.0,
        '[0,0,1/2]Z5-[O9:g:dsp]E(a)': 1.0,
        #'[0,0,1/2]Z5-[Nb1:a:dsp]Eu(b)': 1.0,
        #'[0,0,1/2]Z5-[O1:f:dsp]B3u(a)': 1.0

        #'[0,0,0]GM5+[Ca1:h:dsp]E(a)':1.0,
        #'[0,0,0]GM5+[Ca1:h:dsp]E(b)':1.0,
        #'[0,0,0]GM5-[Cs1:d:dsp]Eu(a)':1.0,
        #'[0,0,0]GM5-[Cs1:d:dsp]Eu(b)':-1.0,
        #'[0,0,0]GM5-[Nb1:a:dsp]Eu(a)':1.0,
        #'[0,0,0]GM5-[Nb1:a:dsp]Eu(b)':1.0,
        #'[0,0,0]GM5-[Nb2:g:dsp]E(a)':1.0,

        #'[1/2,1/2,1/2]A1-[O5:i:dsp]B1(a)':1.0

        #'[1/2,1/2,1/2]A2+[O1:f:dsp]B2u(a)':1.0,
        #'[1/2,1/2,1/2]A2+[O5:i:dsp]B2(a)':1.0,

        #'[1/2,1/2,1/2]A3+[O1:f:dsp]B3u(a)':1.0,
        #'[1/2,1/2,1/2]A3+[O5:i:dsp]B1(a)':1.0,

        #'[1/2,1/2,1/2]A5+[O1:f:dsp]B1u(a)':1.0,
        '[1/2,1/2,1/2]A5+[O1:f:dsp]B1u(b)': 1.0,

        #'[1/2,1/2,0]M1-[O5:i:dsp]B1(a)':1.0,
        #'[1/2,1/2,0]M3+[O1:f:dsp]B3u(a)': 1.0,
        #'[1/2,1/2,0]M3+[O5:i:dsp]B1(a)':1.0,

        #'[1/2,1/2,0]M5+[O1:f:dsp]B1u(a)':1.0,

        #'[0,1/2,0]X3+[O3:g:dsp]E(a)':1.0,
        #'[0,1/2,0]X3+[O3:g:dsp]E(b)':1.0

        #'[0,1/2,1/2]R3+[O1:f:dsp]B1u(a)':1.0,
        #'[0,1/2,1/2]R3+[O1:f:dsp]B1u(b)':1.0,
        #'[0,1/2,1/2]R3+[O3:g:dsp]E(b)': 1.0,
        #'[0,1/2,1/2]R3+[O5:i:dsp]A1(b)':1.0,
    })


def Xa_modes(single_modes):
    return tuple([m] for m in single_modes if m[1] == 'a')


def Xab_modes(single_modes):
    ret = []
    for mode in single_modes:
        if mode[1] == 'a' and (mode[0], 'b') in single_modes:
            ret.append((mode, (mode[0], 'b')))
    return ret


def get_XY_combinations(single_modes):
    Xlist = set(m[0] for m in single_modes)
    return tuple(combinations(Xlist, 2))


def XaYb_modes(single_modes):
    nmodes = len(single_modes)
    XY_comb = get_XY_combinations(single_modes)
    ret = []
    for X, Y in XY_comb:
        if (X, 'a') in single_modes and (Y, 'b') in single_modes:
            ret.append(((X, 'a'), (Y, 'b')))
        elif (Y, 'a') in single_modes and (X, 'b') in single_modes:
            ret.append(((Y, 'a'), (X, 'b')))
    return ret


def XabYa_modes(single_modes):
    Xab_list = Xab_modes(single_modes)
    ret = []
    for Xab in Xab_list:
        for m in single_modes:
            if Xab[0][0] != m[0] and m[1] == 'a':
                ret.append(Xab + (m, ))
    return ret


def XabYab_modes(single_modes):
    Xab_list = Xab_modes(single_modes)
    Xabcomb = combinations(Xab_list, 2)
    ret = list(s[0] + s[1] for s in Xabcomb)
    return ret


def name_modes(modes):
    nlist = tuple('%s_%s' % (m[0], m[1]) for m in modes)
    name = '_'.join(nlist)
    return name


def test_parser():
    myparser = Isomode('./mode_details/A_0.txt')
    #print(myparser.get_all_mode_fullname())
    print(myparser.symbols)
    #myparser.substitute_element({'Cs':'K'})
    #print(myparser.get_mode_displacement("[0,0,1/2]Z5-[O9:g:dsp]E(a)"))
    myparser.read_mode_amplitudes('./mode_details/total.txt')
    myparser.prepare_structure(pickle_fname='all_modes0.03.pickle')
    #myparser.prepare_structure_single(
    #    primitive=True,
    #    cif_dir='single_mode',
    #    pickle_file_name='single_mode.pickle',
    #    amp=0.1, )
    single_modes = sorted(myparser.get_total_modes().keys())
    Xa = Xa_modes(single_modes)
    Xab = Xab_modes(single_modes)
    XaYb = XaYb_modes(single_modes)
    XabYa = XabYa_modes(single_modes)
    XabYab = XabYab_modes(single_modes)
    #print(get_XY_combinations(single_modes))
    for ms in [Xa, Xab, XaYb, XabYa, XabYab]:
        for m in ms:
            #print(name_modes(m))
            pass

    print(name_modes(XabYab[0]))


#test_parser()



import numpy as np
from scipy.optimize import minimize_scalar
from abipy.abilab import abiopen
from pyDFTutils.perovskite.frozen_mode import distorted_cell
from ase import Atoms
from ase.io import write
import spglib.spglib
import matplotlib.pyplot as plt
from functools import partial
from numpy import array


def displacement_cart_to_evec(displ_cart,
                              masses,
                              scaled_positions,
                              qpoint=None,
                              add_phase=True):
    """
    displ_cart: cartisien displacement. (atom1_x, atom1_y, atom1_z, atom2_x, ...)
    masses: masses of atoms.
    scaled_postions: scaled postions of atoms.
    qpoint: if phase needs to be added, qpoint must be given.
    add_phase: whether to add phase to the eigenvectors.
    """
    if add_phase and qpoint is None:
        raise ValueError('qpoint must be given if adding phase is needed')
    m = np.sqrt(np.kron(masses, [1, 1, 1]))
    evec = displ_cart * m
    if add_phase:
        phase = [
            np.exp(-2j * np.pi * np.dot(pos, qpoint))
            for pos in scaled_positions
        ]
        phase = np.kron(phase, [1, 1, 1])
        evec *= phase
        evec /= np.linalg.norm(evec)
    return evec


def rotate_evecs(evecs, theta):
    length = len(evecs)
    if length==3:
        print(length)
    if length != 2:
        return evecs
        #raise NotImplementedError("Only multiplicity=2 rotation implemented. Here multiplicity=%s"%length)
    #R=np.array([[np.cos(theta), -np.sin(theta) ],
    #            [np.sin(theta), np.cos(theta)]])
    #newevec=np.copy(evec)
    #for i in range(length):
    #    newevec[0+i*3:2+i*3]=np.dot( evec[0+i*3:2+i*3],R)
    u = np.array(evecs[0])
    v = np.array(evecs[1])
    newvec1 = np.cos(theta) * u - np.sin(theta) * v
    newvec2 = np.sin(theta) * u + np.cos(theta) * v
    return [newvec1, newvec2]


def project_to_x(vec):
    xs = vec[::3]
    ys = vec[1::3]
    return np.linalg.norm(xs, ord=1) - np.linalg.norm(ys, ord=1)


def test_align(a):
    thetas = np.arange(0, np.pi, 0.01)
    rxs = []
    for theta in thetas:
        #print(theta)
        rx = rotate_evecs(a, theta)[0]
        #print(rx)
        #print(project_to_x(rx))
        rxs.append(project_to_x(rx))
    plt.plot(thetas, rxs)
    plt.show()


rfunc = lambda theta, modes: -project_to_x(rotate_evecs(modes, theta)[0])


def align_degenerate_modes(modes):
    """
    modes are degenerate
    """
    #modes=np.real(modes)
    nmodes = len(modes)
    func = partial(rfunc, modes=modes)
    res = minimize_scalar(func, tol=1e-19)  #, bounds=[0, np.pi])
    theta=res.x
    new_modes=rotate_evecs(modes, theta)
    print("theta: ", theta / np.pi)
    for i in range(len(modes)):
        mi = modes[i] / np.linalg.norm(modes[i])
        nmi = new_modes[i] / np.linalg.norm(new_modes[i])
        #print("Aligning=========\n", np.real(mi).reshape((16,3)), np.real(nmi).reshape((16,3)))
    return new_modes


#
#a= [array([  2.53101288e-02,  -3.82088317e-02,  -4.86461363e-17]), array([  2.46900913e-01,  -3.72728069e-01,  -6.32567296e-18]), array([  2.46900913e-01,  -3.72728069e-01,  -3.08602526e-17]), array([ -7.22928661e-14,  -4.24936302e-02,   1.09275770e-16]), array([  2.13773443e-13,   1.25572159e-01,   1.92532429e-16]), array([  2.13905282e-13,   1.25572159e-01,   2.64991722e-16]), array([  1.45509166e-33,  -3.69736624e-33,  -1.68011693e-32]), array([ -5.86725440e-33,   2.72840948e-33,   2.03582253e-32]), array([ -6.60305144e-13,  -3.87737190e-01,   4.33930585e-17]), array([ -6.59999833e-13,  -3.87737190e-01,   4.13339061e-17]), array([ -4.71852214e-33,  -1.41947245e-32,  -8.36661187e-18]), array([ -1.55464941e-32,   9.19108395e-33,  -1.55779057e-17]), array([  2.25675740e-32,   4.41022949e-32,   8.36661187e-18]), array([  5.47739548e-32,  -1.94551410e-32,   1.55779057e-17]), array([  3.75366405e-13,   2.20589524e-01,   7.57601251e-17]), array([  3.75727227e-13,   2.20589524e-01,   1.29228876e-16])]

a = [
    array([-1.87833463e-18, -3.03705925e-19, 7.35475461e-33]),
    array([-1.29429689e-01, -1.76264658e-02, -8.63292537e-17]),
    array([1.29429689e-01, 1.76264658e-02, -9.54443562e-18]),
    array([8.56027336e-18, 1.28764580e-16, 1.19290208e-16]),
    array([1.25554614e-03, 9.21937203e-03, -3.31492986e-17]),
    array([-1.25554614e-03, -9.21937203e-03, 5.42249759e-17]),
    array([-4.28399726e-33, 8.79432863e-34, 3.40156914e-17]),
    array([6.91766327e-34, 1.94798870e-32, 7.25285576e-18]),
    array([-6.08154119e-02, -4.46562568e-01, 7.35384259e-17]),
    array([6.08154119e-02, 4.46562568e-01, -1.10793322e-17]),
    array([-1.83865500e-16, 5.59979131e-17, 1.78471407e-01]),
    array([-1.09030103e-16, 3.16500949e-16, 2.43052440e-02]),
    array([8.37074694e-16, -1.80417162e-17, -1.78471407e-01]),
    array([4.34089995e-17, -4.88475679e-16, -2.43052440e-02]),
    array([5.01347561e-03, 3.68135391e-02, -3.35885475e-17]),
    array([-5.01347561e-03, -3.68135391e-02, 1.25226772e-17])
]
a=[[1,1,0],
   [1,-1,0]]
#a = np.array(a).flatten()
print(a)
print(rotate_evecs(a, theta=np.pi/4))
print(align_degenerate_modes(a))
test_align(a)

#a=np.array([1,1,0, 1,-1,0],dtype=float)
#test_align()
#print(rotate_evec(a, np.pi/4))
#print(project_to_x(rotate_evec(a, np.pi/4)))
#print(rotate_evec(a, 0.774))
#print(a.reshape(16, 3))
#print(align_one_mode(a))


def align_all_modes(evals, evecs, tol=1e-7):
    """
    Here we assum evals are already sorted.
    """
    multi_modes = []
    multi_modes_inds = []
    new_evals = []
    new_evecs = []
    for i, evec in enumerate(evecs):
        # each time evec[i]!=evec[i-1], deal with mutli_modes and empty it,
        # then push eve[i] into multi_modes
        if np.abs(evals[i - 1] - evals[i]) > tol:
            if len(multi_modes) == 1:  # multi=1, no need to align
                new_evecs.append(multi_modes[0])
            elif len(multi_modes) > 1:  # multi, align and save
                for e in align_degenerate_modes(multi_modes):
                    new_evecs.append(e)
                if len(multi_modes) ==3:  # multi, align and save 
                    print(evals[i-1], evec)
            # clean multi_modes and save new
            multi_modes = []
            multi_modes.append(evec)
        else:
            multi_modes.append(evec)
    return new_evecs


def read_modes(fname='./run.abo_PHBST.nc'):
    qdict = {
        'Gamma': [0.0, 0.0, 0.0],
        'Xy': [0, 0.5, 0],
        'Xx': [0.5, 0.0, 0],
        'M': [0.5, 0.5, 0],
        'Rx': [0.5, 0.0, 0.5],
        'Ry': [0.0, 0.5, 0.5],
        'A': [0.5, 0.5, 0.5],
        'Z': [0, 0, 0.5]
    }
    phbst = abiopen(fname)
    atoms = phbst.structure.to_ase_atoms()
    write('primitive.cif',atoms)
    nbranch = 3 * len(atoms)
    masses = atoms.get_masses()
    scaled_positions = atoms.get_scaled_positions()
    myfile = open('unstable_modes.txt', 'w')
    for qname, qpt in qdict.items():
        #for i in range(nbranch):
        displ_carts = [
            phbst.get_phmode(qpt, i).displ_cart for i in range(nbranch)
        ]
        freqs = [phbst.get_phmode(qpt, i).freq for i in range(nbranch)]
        align_evecs = False
        align_disp = True
        if align_evecs:
            evecs = [
                displacement_cart_to_evec(
                    displ_cart,
                    masses=masses,
                    scaled_positions=scaled_positions,
                    qpoint=qpt,
                    add_phase=True) for displ_cart in displ_carts
            ]

            nevecs = align_all_modes(freqs, evecs)
        elif align_disp:
            ndispl_carts = align_all_modes(freqs, displ_carts)
            nevecs = [
                displacement_cart_to_evec(
                    displ_cart,
                    masses=masses,
                    scaled_positions=scaled_positions,
                    qpoint=qpt,
                    add_phase=True) for displ_cart in ndispl_carts
            ]
        else:
            ndispl_carts = displ_carts
            nevecs = [
                displacement_cart_to_evec(
                    displ_cart,
                    masses=masses,
                    scaled_positions=scaled_positions,
                    qpoint=qpt,
                    add_phase=True) for displ_cart in ndispl_carts
            ]

        for i in range(nbranch):
            freq = freqs[i]

            if freq < 0:
                evec = nevecs[i]
                #displ_cart=displ_carts[i]
                #evec = displacement_cart_to_evec(
                #    displ_cart,
                #    masses=masses,
                #    scaled_positions=scaled_positions,
                #    qpoint=qpt,
                #    add_phase=True)
                #evec=np.real(evec/np.linalg.norm(evec))

                scell = distorted_cell(atoms, supercell_matrix=np.eye(3) * 2)
                disp = scell._get_displacements(
                    evec, qpt, amplitude=1, argument=0)
                #disp=
                newcell = scell._get_cell_with_modulation(disp)
                newcell = Atoms(newcell)
                spacegroup = spglib.get_spacegroup(newcell, symprec=1e-3)
                print(spacegroup)

                write('unstable_modes/%s_%s.cif' % (qname, i), newcell)
                myfile.write("==============================\n")
                #evec=align(evec)
                #evec=np.real(evec/np.linalg.norm(evec))
                #evec=align([evec])[0]
                myfile.write(
                    "Evec: %s\n" % np.real(evec.reshape((16, 3))))
                myfile.write("Freq: %s\n" % (freq))
                myfile.write("%s_%s : %s\n" % (qname, i, spacegroup))

            #print(evec / np.linalg.norm(evec))
    return phbst


read_modes()



#!/usr/bin/env python3
import numpy as np
import os
from isomode.phonmode import phonon_distort_generator
from isodistort_parser import Isomode
from isomode.pydistort import isodistort, view_distort


def run_all(
        fname='./run.abo_PHBST.nc',  # phonon band netcdf file
        qdict={
            'Gamma': [0.0, 0.0, 0.0],
            'Xy': [0, 0.5, 0],
            'Xx': [0.5, 0.0, 0],
            'M': [0.5, 0.5, 0],
            'Rx': [0.5, 0.0, 0.5],
            'Ry': [0.0, 0.5, 0.5],
            'A': [0.5, 0.5, 0.5],
            'Z': [0, 0, 0.5]
        },  # qpoints in netcdf file
        path='tmp',  # temporary directory, but perhaps you may find things useful in it?
        supercell_matrix=np.eye(3) * 2,  # supercell matrix
        max_freq=0.0,  # maximum frequency. use 0.0 if only unstable mode is required
        amp=0.03,  # amplitude of each mode
        pickle_fname='all_modes.pickle',  # output to this pickle file
        cif_dir='all_modes',  # output cif to this directory
        primitive=True  # whether to make it primitve
):
    dg = phonon_distort_generator(fname, qdict)

    # generate primitive cell struture. write to path/primitive.cif
    dg.generate_primitive_structure(path=path)

    # generating single phonon mode structure
    # comment if not needed
    ds = dg.generate_distorted_structures(
        supercell_matrix=supercell_matrix,
        amplitude=0.5,
        unstable_only=True,
        path=os.path.join(path, 'single_modes'))

    # generate structure with all (unstable) phonons.
    # structure is written to
    ds = dg.generate_combined_mode_distorted_structures(
        supercell_matrix=supercell_matrix, max_freq=max_freq, path=path)

    # use isodistort to analyze the structure with all phonons.
    #iso = isodistort(
    #    parent_cif=os.path.join(path, 'primitive.cif'),
    #    distorted_cif=os.path.join(path, 'combined_modes.cif'))
    #ampt = iso.get_mode_amplitude_text()
    #iso.get_mode_amplitude_text()

    iso = view_distort(
        parent_fname=os.path.join(path, 'primitive.cif'),
        distorted_fname=os.path.join(path, 'combined_modes.cif'),
        out_fname=os.path.join(path, 'mode_detail.txt'))

    det_txt = os.path.join(path,'mode_detail.txt')
    #det_txt='mode_detail.txt'
    #mode_details = iso.get_mode_details(save_fname=det_txt)

    # read mode definition and amplitudes from 'mode_detail.txt'
    myparser = Isomode(det_txt)
    myparser.read_mode_amplitudes(det_txt)
    myparser.prepare_structure(
        pickle_fname=pickle_fname,  # file name of pickle file
        cif_dir=cif_dir,  # cif file directory
        amp=amp,  # amplitude
        primitive=primitive# whether to make it primitve
    )

run_all(
        fname='./run.abo_PHBST.nc',  # phonon band netcdf file
        qdict={
            'Gamma': [0.0, 0.0, 0.0],
            'Xy': [0, 0.5, 0],
            'Xx': [0.5, 0.0, 0],
            'M': [0.5, 0.5, 0],
            'Rx': [0.5, 0.0, 0.5],
            'Ry': [0.0, 0.5, 0.5],
            'A': [0.5, 0.5, 0.5],
            'Z': [0, 0, 0.5]
        },  # qpoints in netcdf file
        path='tmp',  # temporary directory, but perhaps you may find things useful in it?
        supercell_matrix=np.eye(3) * 2,  # supercell matrix
        max_freq=0.0,  # maximum frequency. use 0.0 if only unstable mode is required
        amp=0.03,  # amplitude of each mode
        pickle_fname='all_modes.pickle',  # output to this pickle file
        cif_dir='all_modes',  # output cif to this directory
        primitive=True  # whether to make it primitve
)



#!/usr/bin/env python
import argparse
from isomode.pydistort import isocif, isodistort, print_summary, view_distort, view_spacegroup

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--parent", help="parent cif file name")
    parser.add_argument("-d", "--distort", help="distorted cif file name")
    parser.add_argument(
        "-o",
        "--output",
        help="mode details output filename",
        default="mode_detail.txt")
    args = parser.parse_args()


    #raise NotImplementedError(
    #    "This script is not working. "
    #)   

    mode_details=view_distort(
        parent_fname=args.parent,
        distorted_fname=args.distort,
        out_fname=args.output)
    
    print_summary(mode_details)

if __name__=='__main__':
    main()
